# NEON AI (TM) SOFTWARE, Software Development Kit & Application Framework
# All trademark and other rights reserved by their respective owners
# Copyright 2008-2025 Neongecko.com Inc.
# Contributors: Daniel McKnight, Guy Daniels, Elon Gasper, Richard Leeds,
# Regina Bloomstine, Casimiro Ferreira, Andrii Pernatii, Kirill Hrymailo
# BSD-3 License
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS;  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE,  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import unittest

from datetime import datetime
from time import sleep

from dateutil.tz import gettz, tzlocal


class LocationUtilTests(unittest.TestCase):
    def test_get_coordinates_complete(self):
        from neon_utils.location_utils import get_coordinates

        # Complete location
        coords = get_coordinates({"city": "Kirkland", "state": "Washington",
                                  "country": "United States"})
        self.assertIsInstance(coords[0], float)
        self.assertIsInstance(coords[1], float)
        sleep(1)  # maps.co rate-limit

        # No city specified
        coords = get_coordinates({"state": "Washington",
                                  "country": "United States"})
        self.assertIsInstance(coords[0], float)
        self.assertIsInstance(coords[1], float)
        sleep(1)  # maps.co rate-limit

        # No state specified
        coords = get_coordinates({"city": "Seattle",
                                  "country": "United States"})
        self.assertIsInstance(coords[0], float)
        self.assertIsInstance(coords[1], float)
        sleep(1)  # maps.co rate-limit

        # No country specified
        coords = get_coordinates({"state": "Washington", "city": "Renton"})
        self.assertIsInstance(coords[0], float)
        self.assertIsInstance(coords[1], float)
        sleep(1)  # maps.co rate-limit

    def test_get_location_from_coords(self):
        from neon_utils.location_utils import get_location
        lat = 47.6038321
        lng = -122.3300624
        location = get_location(lat, lng)
        self.assertEqual(len(location), 4)
        self.assertEqual(location, ("Seattle", "King County", "Washington",
                                    "United States"))

        # Test 'hamlet' location
        location = get_location(34.46433387046654, -81.99487538579375)
        self.assertIsInstance(location[0], str)

    def test_get_timezone_from_coords(self):
        from neon_utils.location_utils import get_timezone
        lat = 47.6038321
        lng = -122.3300624
        timezone, offset = get_timezone(lat, lng)
        self.assertIsInstance(timezone, str)
        self.assertEqual(timezone, "America/Los_Angeles")
        self.assertIsInstance(offset, float)
        self.assertIn(offset, (-7.0, -8.0))

        lat = 35.0000
        lon = 103.000
        timezone, offset = get_timezone(lat, lon)
        self.assertEqual(timezone, "Asia/Shanghai")
        self.assertEqual(offset, 8.0)

    def test_to_system_time(self):
        from neon_utils.location_utils import to_system_time
        tz_aware_dt = datetime.now(gettz("America/NewYork"))
        system_dt = to_system_time(tz_aware_dt)
        self.assertEqual(system_dt.tzinfo, tzlocal())
        self.assertEqual(tz_aware_dt.timestamp(), system_dt.timestamp())

        tz_aware_dt = datetime.now(gettz("America/Los_Angeles"))
        system_dt = to_system_time(tz_aware_dt)
        self.assertEqual(system_dt.tzinfo, tzlocal())
        self.assertEqual(tz_aware_dt.timestamp(), system_dt.timestamp())

        tz_naiive_dt = datetime.now()
        system_dt = to_system_time(tz_naiive_dt)
        self.assertEqual(system_dt.tzinfo, tzlocal())
        self.assertEqual(tz_naiive_dt.timestamp(), system_dt.timestamp())

    def test_get_full_location(self):
        from neon_utils.location_utils import get_full_location
        location_en = get_full_location("Seattle, Washington")
        self.assertIsInstance(location_en, dict)
        self.assertIsInstance(location_en['lat'], str)
        self.assertIsInstance(location_en['lon'], str)
        self.assertIsInstance(location_en['address'], dict)
        self.assertEqual(location_en['address']['city'], "Seattle")
        self.assertEqual(location_en['address']['state'], "Washington")
        self.assertEqual(location_en['address']['country'], "United States")
        self.assertEqual(location_en['address']['country_code'], "us")
        sleep(1)  # maps.co rate-limit

        location_es = get_full_location("Seattle, Washington", "es")
        self.assertAlmostEqual(float(location_es['lat']),
                               float(location_en['lat']), places=6)
        self.assertAlmostEqual(float(location_es['lon']),
                               float(location_en['lon']), places=6)
        self.assertEqual(location_es['address']['country'],
                         "Estados Unidos de América")
        self.assertEqual(location_en['address']['country_code'], "us")
        sleep(1)  # maps.co rate-limit

        location_from_coords = get_full_location((location_en['lat'],
                                                  location_en['lon']))
        self.assertAlmostEqual(float(location_from_coords['lat']),
                               float(location_en['lat']), 3)
        self.assertAlmostEqual(float(location_from_coords['lon']),
                               float(location_en['lon']), 3)
        self.assertEqual(location_from_coords['address']['city'], "Seattle")
        self.assertEqual(location_from_coords['address']['state'],
                         "Washington")
        self.assertEqual(location_from_coords['address']['country'],
                         "United States")
        self.assertEqual(location_from_coords['address']['country_code'], "us")

    def test_set_nominatim_domain(self):
        from neon_utils.location_utils import set_nominatim_domain, \
            _NOMINATIM_DOMAIN
        self.assertIsInstance(_NOMINATIM_DOMAIN, str)

        set_nominatim_domain("https://geocode.maps.co")
        from neon_utils.location_utils import _NOMINATIM_DOMAIN
        self.assertEqual(_NOMINATIM_DOMAIN, "geocode.maps.co")

        set_nominatim_domain("nominatim.openstreetmap.org")
        from neon_utils.location_utils import _NOMINATIM_DOMAIN
        self.assertEqual(_NOMINATIM_DOMAIN, "nominatim.openstreetmap.org")


if __name__ == '__main__':
    unittest.main()



# NEON AI (TM) SOFTWARE, Software Development Kit & Application Framework
# All trademark and other rights reserved by their respective owners
# Copyright 2008-2025 Neongecko.com Inc.
# Contributors: Daniel McKnight, Guy Daniels, Elon Gasper, Richard Leeds,
# Regina Bloomstine, Casimiro Ferreira, Andrii Pernatii, Kirill Hrymailo
# BSD-3 License
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS;  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE,  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import requests
import json

from typing import Optional
from os import makedirs, remove
from os.path import join, isfile, isdir, dirname
from time import time
from ovos_utils.log import LOG
from ovos_utils.xdg_utils import xdg_cache_home

_DEFAULT_BACKEND_URL = None
_client_config = {}
_headers = {}


def set_default_backend_url(url: Optional[str] = None):
    """
    Set the default backend URL
    @param url: HANA backend url to use, else read from configuration
    """
    global _DEFAULT_BACKEND_URL
    if not url:
        from ovos_config.config import Configuration
        url = Configuration().get('hana', {}).get('url') or \
            "https://hana.neonaiservices.com"
    if url and url != _DEFAULT_BACKEND_URL:
        LOG.info(f"Updating HANA backend URL to {url}")
        _DEFAULT_BACKEND_URL = url
        global _client_config
        global _headers
        _client_config = {}
        _headers = {}


def _get_client_config_path(url: str):
    url_key = url.split('/')[2]
    return join(xdg_cache_home(), "neon", f"hana_token_{url_key}.json")


class ServerException(Exception):
    """Exception class representing a backend server communication error"""


def _init_client(backend_address: str, ssl_verify: bool = True):
    """
    Initialize request headers for making backend requests. If a local cache is
    available it will be used, otherwise an auth request will be made to the
    specified backend server
    @param backend_address: Hana server URL to connect to
    """
    global _client_config
    global _headers
    # TODO: Config on disk could be invalid here; consider validating tokens now
    #   instead of waiting for exception handling with a failed request
    if not _client_config:
        client_config_path = _get_client_config_path(backend_address)
        if isfile(client_config_path):
            with open(client_config_path) as f:
                _client_config = json.load(f)
        else:
            _get_token(backend_address, ssl_verify=ssl_verify)

    if not _headers:
        _headers = {"Authorization": f"Bearer {_client_config['access_token']}"}


def _get_token(backend_address: str, username: str = "guest",
               password: str = "password",
               ssl_verify: bool = True):
    """
    Get new auth tokens from the specified server. This will cache the returned
    token, overwriting any previous data at the cache path.
    @param backend_address: Hana server URL to connect to
    @param username: Username to authorize
    @param password: Password for specified username
    """
    global _client_config
    # TODO: username/password from configuration
    resp = requests.post(f"{backend_address}/auth/login",
                         json={"username": username,
                               "password": password},
                         verify=ssl_verify)
    if not resp.ok:
        raise ServerException(f"Error logging into {backend_address}. "
                              f"{resp.status_code}: {resp.text}")
    _client_config = resp.json()
    client_config_path = _get_client_config_path(backend_address)
    if not isdir(dirname(client_config_path)):
        makedirs(dirname(client_config_path))
    with open(client_config_path, "w+") as f:
        json.dump(_client_config, f, indent=2)


def _refresh_token(backend_address: str, ssl_verify: bool = True):
    """
    Get new tokens from the specified server using an existing refresh token
    (if it exists). This will update the cached tokens and associated metadata.
    @param backend_address: Hana server URL to connect to
    """
    global _client_config
    _init_client(backend_address)
    update = requests.post(f"{backend_address}/auth/refresh", json={
        "access_token": _client_config.get("access_token"),
        "refresh_token": _client_config.get("refresh_token"),
        "client_id": _client_config.get("client_id")},
        verify=ssl_verify)
    if not update.ok:
        raise ServerException(f"Error updating token from {backend_address}. "
                              f"{update.status_code}: {update.text}")
    _client_config = update.json()

    # Update request headers with new token
    global _headers
    _headers['Authorization'] = f"Bearer {_client_config['access_token']}"

    client_config_path = _get_client_config_path(backend_address)
    with open(client_config_path, "w+") as f:
        json.dump(_client_config, f, indent=2)


def request_backend(endpoint: str, request_data: dict,
                    server_url: str = _DEFAULT_BACKEND_URL,
                    ssl_verify: bool = True) -> dict:
    """
    Make a request to a Hana backend server and return the json response
    @param endpoint: server endpoint to query
    @param request_data: dict data to send in request body
    @param server_url: Base URL of Hana server to query
    @param ssl_verify: If False, disables SSL verification
    @returns: dict response
    """
    global _client_config
    global _headers
    if not server_url:
        set_default_backend_url()
        server_url = _DEFAULT_BACKEND_URL
    if server_url != _DEFAULT_BACKEND_URL and _client_config:
        LOG.info(f"Using new remote: {server_url}")
        _client_config = {}
        _headers = {}
    _init_client(server_url, ssl_verify=ssl_verify)
    if _client_config.get("expiration", 0) - time() < 30:
        try:
            _refresh_token(server_url, ssl_verify=ssl_verify)
        except ServerException as e:
            LOG.error(e)
            _get_token(server_url, ssl_verify=ssl_verify)
    request_kwargs = {"url": f"{server_url}/{endpoint.lstrip('/')}",
                      "json": request_data, "headers": _headers,
                      "verify": ssl_verify}
    resp = requests.post(**request_kwargs)
    if resp.status_code == 502:
        # This is raised occasionally on valid requests. Need to resolve in HANA
        resp = requests.post(**request_kwargs)
    if resp.ok:
        return resp.json()
    else:
        try:
            error = resp.json()["detail"]
            # Token is actually expired, refresh and retry
            if error == "Invalid or expired token.":
                LOG.warning(f"Token is expired. time={time()}|"
                            f"expiration={_client_config.get('expiration')}")
                _refresh_token(server_url, ssl_verify=ssl_verify)
                resp = requests.post(**request_kwargs)
                if resp.ok:
                    return resp.json()
        except Exception as e:
            LOG.error(e)
            # Clear cached config to force re-evaluation on next request
            _client_config = {}
            _headers = {}
            if resp.status_code == 403:
                # Invalid token supplied; remove it from local cache
                config_file = _get_client_config_path(server_url)
                if isfile(config_file):
                    LOG.warning(f"Removing invalid token cache: {config_file}")
                    remove(config_file)
        raise ServerException(f"Error response {resp.status_code}: {resp.text}")



# NEON AI (TM) SOFTWARE, Software Development Kit & Application Framework
# All trademark and other rights reserved by their respective owners
# Copyright 2008-2025 Neongecko.com Inc.
# Contributors: Daniel McKnight, Guy Daniels, Elon Gasper, Richard Leeds,
# Regina Bloomstine, Casimiro Ferreira, Andrii Pernatii, Kirill Hrymailo
# BSD-3 License
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS;  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE,  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import inspect

from time import time, sleep
from typing import Optional, Callable

from ovos_bus_client import MessageBusClient, Message
from ovos_utils.log import log_deprecation

from neon_utils.logger import LOG

try:
    from mock.mock import Mock
except ImportError:
    raise ImportError("`mock` is not installed,"
                      " pip install neon-utils[signal]")
_BUS: Optional[MessageBusClient] = None
_MAX_TIMEOUT: Optional[int] = None

_create_signal: Optional[Callable] = None
_check_for_signal: Optional[Callable] = None
_wait_for_signal_clear: Optional[Callable] = None
_wait_for_signal_create: Optional[Callable] = None


def create_signal(*args, **kwargs):
    global _create_signal
    if not _create_signal:
        LOG.warning("create_signal called before signal manager init")
        init_signal_handlers()
    return _create_signal(*args, **kwargs)


def check_for_signal(*args, **kwargs):
    global _check_for_signal
    if not _check_for_signal:
        LOG.warning("check_for_signal called before signal manager init")
        init_signal_handlers()
    return _check_for_signal(*args, **kwargs)


def wait_for_signal_clear(*args, **kwargs):
    global _wait_for_signal_clear
    if not _wait_for_signal_clear:
        LOG.warning("wait_for_signal_clear called before signal manager init")
        init_signal_handlers()
    return _wait_for_signal_clear(*args, **kwargs)


def wait_for_signal_create(*args, **kwargs):
    global _wait_for_signal_create
    if not _wait_for_signal_create:
        LOG.warning("wait_for_signal_create called before signal manager init")
        init_signal_handlers()
    return _wait_for_signal_create(*args, **kwargs)


def init_signal_bus(bus: MessageBusClient):
    """
    Specify a MessageBusClient to use for methods in this module
    :param bus: Connected and Running MessageBusClient
    """
    global _BUS
    if not bus.started_running:
        bus.run_in_thread()
    _BUS = bus


def init_signal_handlers():
    """
    Initialize the proper signal methods dependent on the Signal Manager
    being available. Any previously imported methods will not be affected by
    calls to this method, but imports of `signal_utils` are.
    i.e. `from neon_utils.signal_utils import check_for_signal`,
     `check_for_signal` is not changed.
     `import neon_utils.signal_utils`,
     `neon_utils.signal_utils.check_for_signal` is changed.
    """
    global _create_signal
    global _check_for_signal
    global _wait_for_signal_clear
    global _wait_for_signal_create
    global _MAX_TIMEOUT
    from ovos_config.config import Configuration
    signal_config = Configuration().get("signal") or dict()
    patch_imports = signal_config.get("patch_imports", True)
    _MAX_TIMEOUT = int(signal_config.get("max_wait_seconds") or 300)

    if check_signal_manager_available():
        LOG.info("Signal Manager Available")
        _create_signal = _manager_create_signal
        _check_for_signal = _manager_check_for_signal
        _wait_for_signal_clear = _manager_wait_for_signal_clear
        _wait_for_signal_create = _manager_wait_for_signal_create

        if patch_imports:
            log_deprecation("Import patching will be deprecated. Disable in "
                            "configuration by setting `signal`.`patch_imports` "
                            "to `False`", "2.0.0")
            import ovos_utils.signal
            ovos_utils.signal.check_for_signal = _check_for_signal
            ovos_utils.signal.create_signal = _create_signal
            try:
                import mycroft.util.signal
                mycroft.util.signal.create_signal = _create_signal
                mycroft.util.signal.check_for_signal = _check_for_signal
                LOG.info(f"Overrode mycroft.util.signal methods")
            except (ImportError, AttributeError) as e:
                LOG.debug(e)
            except TypeError as e:
                # This comes from tests overriding MessageBusClient()
                LOG.error(e)

    else:
        LOG.warning("FS signals are deprecated. Signal methods will have no effect.")
        if patch_imports:
            log_deprecation("Import patching will be deprecated. Disable in "
                            "configuration by setting `signal`.`patch_imports` "
                            "to `False`", "2.0.0")
            import ovos_utils.signal
            _create_signal = ovos_utils.signal.create_signal
            _check_for_signal = ovos_utils.signal.check_for_signal
            _wait_for_signal_clear = _fs_wait_for_signal_clear
            _wait_for_signal_create = _fs_wait_for_signal_create
        else:
            _create_signal = Mock(return_value=False)
            _check_for_signal = Mock(return_value=False)
            _wait_for_signal_clear = Mock(return_value=False)
            _wait_for_signal_create = Mock(return_value=False)


def check_signal_manager_available() -> bool:
    """
    Method to check if a signal manager service is available
    """
    global _BUS
    if not _BUS:
        LOG.warning("Initializing new messagebus connection")
        init_signal_bus(MessageBusClient())
    if _BUS.connected_event.wait(10):  # Wait up to 10 seconds for the bus service
        response = _BUS.wait_for_response(Message("neon.signal_manager_active"))
        LOG.debug(f"signal_manager_active={response is not None}")
        return response is not None
    LOG.error(f"Signal manager check gave up waiting for the MessageBus")
    return False


def _manager_create_signal(signal_name: str, *_, **__) -> bool:
    """
    Backwards-compatible method for creating a signal
    :param signal_name: named signal to create
    :return: True if signal exists
    """
    call = inspect.stack()[2]
    module = inspect.getmodule(call.frame)
    name = module.__name__ if module else call.filename
    stat = _BUS.wait_for_response(Message("neon.create_signal",
                                          {"signal_name": signal_name},
                                          {"origin_module": name,
                                           "origin_line": call.lineno}),
                                  f"neon.create_signal.{signal_name}", 10) or \
        Message('')
    return stat.data.get("is_set")


def _manager_check_for_signal(signal_name: str, sec_lifetime: int = 0, *_, **__) -> bool:
    """
    Backwards-compatible method for checking for a signal
    :param signal_name: name of signal to check
    :param sec_lifetime: max age of signal in seconds before clearing it and
        returning False
    :return: True if signal exists
    """
    call = inspect.stack()[2]
    module = inspect.getmodule(call.frame)
    name = module.__name__ if module else call.filename
    stat = _BUS.wait_for_response(Message("neon.check_for_signal",
                                          {"signal_name": signal_name,
                                           "sec_lifetime": sec_lifetime},
                                          {"origin_module": name,
                                           "origin_line": call.lineno}),
                                  f"neon.check_for_signal.{signal_name}",
                                  10) or Message('')
    return stat.data.get("is_set")


def _manager_wait_for_signal_create(signal_name: str,
                                    timeout: int = 30) -> bool:
    """
    Block until the specified signal is set or timeout is reached
    :param signal_name: name of signal to check
    :param timeout: max seconds to wait for signal to be created,
        Default is 30 seconds
    :return: True if signal exists
    """
    timeout = _MAX_TIMEOUT if timeout > _MAX_TIMEOUT else timeout  # Cap wait
    bus_wait_time = timeout + 5  # Allow some padding for bus handler
    stat = _BUS.wait_for_response(Message("neon.wait_for_signal_create",
                                          {"signal_name": signal_name,
                                           "timeout": timeout}),
                                  f"neon.wait_for_signal_create.{signal_name}",
                                  bus_wait_time)
    return stat.data.get("is_set")


def _manager_wait_for_signal_clear(signal_name: str, timeout: int = 30) -> bool:
    """
    Block until the specified signal is cleared or timeout is reached
    :param signal_name: name of signal to check
    :param timeout: max seconds to wait for signal to be created,
        Default is 30 seconds
    :return: True if signal exists
    """
    timeout = _MAX_TIMEOUT if timeout > _MAX_TIMEOUT else timeout  # Cap wait
    bus_wait_time = timeout + 5  # Allow some padding for bus handler
    stat = _BUS.wait_for_response(Message("neon.wait_for_signal_clear",
                                          {"signal_name": signal_name,
                                           "timeout": timeout}),
                                  f"neon.wait_for_signal_clear.{signal_name}",
                                  bus_wait_time)
    return stat.data.get("is_set")


def _fs_wait_for_signal_create(signal_name: str, timeout: int = 30):
    expiration = time() + timeout
    while not check_for_signal(signal_name, -1) and time() < expiration:
        sleep(0.1)
    return check_for_signal(signal_name, -1)


def _fs_wait_for_signal_clear(signal_name: str, timeout: int = 30):
    expiration = time() + timeout
    while check_for_signal(signal_name, -1) and time() < expiration:
        sleep(0.1)
    return check_for_signal(signal_name, -1)



# NEON AI (TM) SOFTWARE, Software Development Kit & Application Framework
# All trademark and other rights reserved by their respective owners
# Copyright 2008-2025 Neongecko.com Inc.
# Contributors: Daniel McKnight, Guy Daniels, Elon Gasper, Richard Leeds,
# Regina Bloomstine, Casimiro Ferreira, Andrii Pernatii, Kirill Hrymailo
# BSD-3 License
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS;  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE,  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from collections import namedtuple
from ovos_bus_client import Message
from neon_utils.messagebus_utils import get_messagebus
try:
    from lingua_franca.internal import get_supported_langs
except ImportError:
    get_supported_langs = None


_bus = None
_default_lang = 'en'

SupportedLanguages = namedtuple('SupportedLanguages', ['stt', 'tts', 'skills'])


def _get_messagebus():
    global _bus
    if not _bus:
        _bus = get_messagebus()
    return _bus


def get_supported_languages(translation: bool = True) -> SupportedLanguages:
    """
    Get a dict of language support information.
    STT and TTS lists include languages that may not be supported by skills
    :param translation: Include skills languages supported via translation
    :returns: SupportedLanguages object with stt, tts, and skills language sets
    """
    return SupportedLanguages(stt=get_supported_input_langs(False),
                              tts=get_supported_output_langs(False),
                              skills=get_supported_skills_langs(translation))


def get_supported_skills_langs(translation: bool = True) -> set:
    """
    Get core language support for skills processing. This is the intersection
    of skills supported langs and Lingua Franca supported langs.
    :param translation: Include languages supported via translation
    :returns: set of languages supported in skills processing
    """
    lf_langs = set(get_supported_langs()) if get_supported_langs else set()
    msg = _get_messagebus().wait_for_response(Message("neon.languages.skills"))
    if not msg:
        langs = [_default_lang]
    elif translation:
        langs = msg.data.get('skill_langs')
    else:
        langs = msg.data.get('native_langs')
    langs = (lang.split('-')[0] for lang in langs)
    return lf_langs & set(langs) if lf_langs and not translation else set(langs)


def get_supported_input_langs(skill_support: bool = True,
                              translation: bool = True,
                              skills_langs: set = None) -> set:
    """
    Get the set of supported input languages
    :param skill_support: Only include languages also supported by skills
    :param translation: Include languages with skill support via translation
    :param skills_langs: Optional set of supported skills languages
    """
    msg = _get_messagebus().wait_for_response(Message("ovos.languages.stt"))
    stt_langs = set(lang.split('-')[0] for lang in msg.data.get('langs')) if \
        msg else set()
    if not skill_support:
        return stt_langs

    skills = skills_langs or get_supported_skills_langs(translation)
    return stt_langs & skills if stt_langs else {_default_lang}


def get_supported_output_langs(skill_support: bool = True,
                               translation: bool = True,
                               skills_langs: set = None) -> set:
    """
    Get the set of supported output languages
        :param skill_support: Only include languages also supported by skills
    :param translation: Include languages with skill support via translation
    :param skills_langs: Optional set of supported skills languages
    """
    msg = _get_messagebus().wait_for_response(Message("ovos.languages.tts"))
    tts_langs = set(lang.split('-')[0] for lang in msg.data.get('langs')) if \
        msg else set()
    if not skill_support:
        return tts_langs

    skills = skills_langs or get_supported_skills_langs(translation)
    return tts_langs & skills if tts_langs else {_default_lang}



# NEON AI (TM) SOFTWARE, Software Development Kit & Application Framework
# All trademark and other rights reserved by their respective owners
# Copyright 2008-2025 Neongecko.com Inc.
# Contributors: Daniel McKnight, Guy Daniels, Elon Gasper, Richard Leeds,
# Regina Bloomstine, Casimiro Ferreira, Andrii Pernatii, Kirill Hrymailo
# BSD-3 License
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS;  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE,  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
import importlib
import shutil
import sys
import os
import unittest
import json
import yaml
import mock

from copy import deepcopy
from time import sleep
from glob import glob
from os.path import join, dirname, isdir, basename, isfile, expanduser, \
    getmtime

sys.path.append(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
from neon_utils.log_utils import LOG

ROOT_DIR = os.path.dirname(os.path.realpath(__file__))
CONFIG_PATH = os.path.join(ROOT_DIR, "configuration")

TEST_DICT = {"section 1": {"key1": "val1",
                           "key2": "val2"},
             "section 2": {"key_1": "val1",
                           "key_2": "val2"}}


class NGIConfigTests(unittest.TestCase):
    def doCleanups(self) -> None:
        if os.getenv("NEON_CONFIG_PATH"):
            os.environ.pop("NEON_CONFIG_PATH")
        for file in glob(os.path.join(CONFIG_PATH, ".*.lock")):
            os.remove(file)
        for file in glob(os.path.join(CONFIG_PATH, ".*.tmp")):
            os.remove(file)
        for file in glob(os.path.join(ROOT_DIR, "credentials", ".*.lock")):
            os.remove(file)
        for file in glob(os.path.join(ROOT_DIR, "credentials", ".*.tmp")):
            os.remove(file)
        if os.path.exists(os.path.join(CONFIG_PATH, "old_user_info.yml")):
            os.remove(os.path.join(CONFIG_PATH, "old_user_info.yml"))

    def test_load_config(self):
        from neon_utils.configuration_utils import NGIConfig
        local_conf = NGIConfig("ngi_local_conf", CONFIG_PATH)
        self.assertIsInstance(local_conf.content, dict)
        self.assertIsInstance(local_conf.content["devVars"], dict)
        self.assertIsInstance(local_conf.content["prefFlags"]["devMode"], bool)

    def test_config_get(self):
        from neon_utils.configuration_utils import NGIConfig
        local_conf = NGIConfig("ngi_local_conf", CONFIG_PATH)
        self.assertIsInstance(local_conf, NGIConfig)
        self.assertIsInstance(local_conf.content, dict)
        self.assertIsInstance(local_conf["devVars"], dict)
        self.assertIsInstance(local_conf["prefFlags"]["devMode"], bool)
        self.assertEqual(local_conf["prefFlags"], local_conf.get("prefFlags"))
        self.assertIsNone(local_conf.get("fake_key"))
        self.assertTrue(local_conf.get("fake_key", True))

    def test_config_set(self):
        from neon_utils.configuration_utils import NGIConfig
        local_conf = NGIConfig("ngi_local_conf", CONFIG_PATH)
        self.assertIsInstance(local_conf.content, dict)
        self.assertIsInstance(local_conf["prefFlags"]["devMode"], bool)

        local_conf["prefFlags"]["devMode"] = True
        self.assertTrue(local_conf["prefFlags"]["devMode"])

        local_conf["prefFlags"]["devMode"] = False
        self.assertFalse(local_conf["prefFlags"]["devMode"])

    def test_get_config_dir_invalid_override(self):
        from neon_utils.configuration_utils import get_config_dir
        os.environ["NEON_CONFIG_PATH"] = "/invalid"
        config_path = get_config_dir()
        self.assertNotEqual(config_path, "/invalid")
        os.environ.pop("NEON_CONFIG_PATH")
        self.assertIsNone(os.getenv("NEON_CONFIG_PATH"))

    def test_make_equal_keys(self):
        from neon_utils.configuration_utils import NGIConfig
        old_user_info = os.path.join(CONFIG_PATH, "old_user_info.yml")
        ngi_user_info = os.path.join(CONFIG_PATH, "ngi_user_info.yml")
        shutil.copy(ngi_user_info, old_user_info)
        user_conf = NGIConfig("ngi_user_info", CONFIG_PATH, True)
        self.assertEqual(user_conf.content["user"]["full_name"], 'Test User')
        self.assertNotIn("phone_verified", user_conf.content["user"])
        self.assertIn('bad_key', user_conf.content["user"])

        user_conf.make_equal_by_keys(NGIConfig("clean_user_info",
                                               CONFIG_PATH).content)
        self.assertIn("phone_verified", user_conf.content["user"])
        self.assertNotIn('bad_key', user_conf.content["user"])
        self.assertFalse(user_conf.content["user"]["phone_verified"])
        self.assertEqual(user_conf.content["user"]["full_name"], 'Test User')

        new_user_info = NGIConfig("ngi_user_info", CONFIG_PATH)
        self.assertEqual(user_conf.content, new_user_info.content)
        shutil.copy(old_user_info, ngi_user_info)

    def test_make_equal_keys_no_recurse(self):
        from neon_utils.configuration_utils import NGIConfig
        skill_settings = NGIConfig("skill_populated", CONFIG_PATH)
        correct_settings = deepcopy(skill_settings.content)
        skill_settings.make_equal_by_keys(NGIConfig("skill_default",
                                                    CONFIG_PATH).content,
                                          False)
        self.assertEqual(correct_settings, skill_settings.content)

    def test_update_keys(self):
        from neon_utils.configuration_utils import NGIConfig
        old_user_info = os.path.join(CONFIG_PATH, "old_user_info.yml")
        ngi_user_info = os.path.join(CONFIG_PATH, "ngi_user_info.yml")
        shutil.copy(ngi_user_info, old_user_info)
        user_conf = NGIConfig("ngi_user_info", CONFIG_PATH, True)
        self.assertEqual(user_conf.content["user"]["full_name"], 'Test User')
        self.assertNotIn("phone_verified", user_conf.content["user"])
        self.assertIn('bad_key', user_conf.content["user"])

        user_conf.update_keys(NGIConfig("clean_user_info",
                                        CONFIG_PATH).content)
        self.assertIn("phone_verified", user_conf.content["user"])
        self.assertIn('bad_key', user_conf.content["user"])
        self.assertFalse(user_conf.content["user"]["phone_verified"])
        self.assertEqual(user_conf.content["user"]["full_name"], 'Test User')

        new_user_info = NGIConfig("ngi_user_info", CONFIG_PATH)
        self.assertEqual(user_conf.content, new_user_info.content)
        shutil.move(old_user_info, ngi_user_info)

    def test_update_key(self):
        from neon_utils.configuration_utils import NGIConfig
        old_user_info = os.path.join(CONFIG_PATH, "old_user_info.yml")
        ngi_user_info = os.path.join(CONFIG_PATH, "ngi_user_info.yml")
        shutil.copy(ngi_user_info, old_user_info)
        user_conf = NGIConfig("ngi_user_info", CONFIG_PATH)

        self.assertEqual(user_conf.content["user"]["full_name"], 'Test User')
        user_conf.update_yaml_file("user", "full_name", "New Name",
                                   multiple=False, final=True)
        self.assertEqual(user_conf.content["user"]["full_name"], 'New Name')
        new_user_conf = NGIConfig("ngi_user_info", CONFIG_PATH)
        self.assertEqual(user_conf.content["user"]["full_name"],
                         new_user_conf.content["user"]["full_name"])
        shutil.copy(old_user_info, ngi_user_info)

    def test_export_json(self):
        import json
        from neon_utils.configuration_utils import NGIConfig
        user_conf = NGIConfig("ngi_user_info", CONFIG_PATH)
        json_file = user_conf.export_to_json()
        with open(json_file, "r") as f:
            from_disk = json.load(f)
        self.assertEqual(from_disk, user_conf.content)
        os.remove(json_file)

    def test_import_dict(self):
        from neon_utils.configuration_utils import NGIConfig
        test_conf = NGIConfig("test_conf", CONFIG_PATH).from_dict(TEST_DICT)
        self.assertEqual(test_conf.content, TEST_DICT)
        from_disk = NGIConfig("test_conf", CONFIG_PATH)
        self.assertEqual(from_disk.content, test_conf.content)
        os.remove(test_conf.file_path)

    def test_import_json(self):
        from neon_utils.configuration_utils import NGIConfig
        from ovos_utils.json_helper import load_commented_json
        json_path = os.path.join(CONFIG_PATH, "mycroft.conf")
        test_conf = NGIConfig("mycroft", CONFIG_PATH).from_json(json_path)
        parsed_json = load_commented_json(json_path)
        self.assertEqual(parsed_json, test_conf.content)
        from_disk = NGIConfig("mycroft", CONFIG_PATH)
        self.assertEqual(from_disk.content, test_conf.content)
        os.remove(test_conf.file_path)

    def test_config_cache(self):
        from neon_utils.configuration_utils import NGIConfig
        from neon_utils.configuration_utils import NGIConfig as NGIConf2
        bak_local_conf = os.path.join(CONFIG_PATH, "ngi_local_conf.bak")
        ngi_local_conf = os.path.join(CONFIG_PATH, "ngi_local_conf.yml")

        shutil.copy(ngi_local_conf, bak_local_conf)
        config_1 = NGIConfig("ngi_local_conf", CONFIG_PATH)
        self.assertFalse(config_1.requires_reload)
        config_2 = NGIConf2("ngi_local_conf", CONFIG_PATH, True)
        self.assertFalse(config_2.requires_reload)
        self.assertEqual(config_1._content, config_2._content)
        self.assertNotEqual(config_1, config_2)

        config_1.update_yaml_file("prefFlags", "autoStart", False)
        self.assertFalse(config_1._pending_write)
        self.assertEqual(config_2._content["prefFlags"]["autoStart"], True)
        self.assertFalse(config_2._pending_write)

        self.assertNotEqual(config_1._loaded, config_2._loaded)
        self.assertGreater(config_1._loaded, config_2._loaded)
        self.assertTrue(config_2.requires_reload)
        self.assertEqual(config_1.content, config_2.content)
        self.assertEqual(config_1._loaded, config_2._loaded)

        config_2.update_yaml_file("prefFlags", "devMode", False, multiple=True)
        self.assertFalse(config_2["prefFlags"]["devMode"])
        self.assertTrue(config_2._pending_write)
        config_2.write_changes()
        self.assertFalse(config_2._pending_write)
        self.assertTrue(config_1.requires_reload)
        self.assertEqual(config_1.content["prefFlags"]["devMode"], False)

        config_2.update_yaml_file("prefFlags", "devMode", True, multiple=True)
        self.assertTrue(config_2._pending_write)
        self.assertTrue(config_2["prefFlags"]["devMode"])
        self.assertFalse(config_1["prefFlags"]["devMode"])

        shutil.move(bak_local_conf, ngi_local_conf)

    def test_multi_config(self):
        from neon_utils.configuration_utils import NGIConfig
        bak_local_conf = os.path.join(CONFIG_PATH, "ngi_local_conf.bak")
        ngi_local_conf = os.path.join(CONFIG_PATH, "ngi_local_conf.yml")

        shutil.copy(ngi_local_conf, bak_local_conf)

        config_objects = []
        for i in range(100):
            config_objects.append(NGIConfig("ngi_local_conf",
                                            CONFIG_PATH, True))

        first_config = config_objects[0]
        last_config = config_objects[-1]
        self.assertIsInstance(first_config, NGIConfig)
        self.assertIsInstance(last_config, NGIConfig)

        self.assertEqual(first_config.content, last_config.content)
        first_config.update_yaml_file("prefFlags", "devMode", False)

        self.assertFalse(last_config["prefFlags"]["devMode"])
        self.assertEqual(first_config.content, last_config.content)

        shutil.move(bak_local_conf, ngi_local_conf)

    def test_concurrent_config_read(self):
        from neon_utils.configuration_utils import NGIConfig
        from threading import Thread
        valid_config = NGIConfig("dep_user_info", CONFIG_PATH)
        test_results = {}

        def _open_config(idx):
            from neon_utils.configuration_utils import NGIConfig as Config
            config = Config("dep_user_info", CONFIG_PATH, True)
            test_results[idx] = config.content == valid_config.content

        for i in range(10):
            Thread(target=_open_config, args=(i,), daemon=True).start()
        while not len(test_results.keys()) == 10:
            sleep(0.5)
        self.assertTrue(all(test_results.values()))

    def test_new_ngi_config(self):
        from neon_utils.configuration_utils import NGIConfig
        config = NGIConfig("temp_conf", CONFIG_PATH)
        self.assertIsInstance(config.content, dict)
        os.remove(os.path.join(CONFIG_PATH, "temp_conf.yml"))


class ConfigurationUtilTests(unittest.TestCase):
    def doCleanups(self) -> None:
        if os.getenv("NEON_CONFIG_PATH"):
            os.environ.pop("NEON_CONFIG_PATH")
        for file in glob(os.path.join(CONFIG_PATH, ".*.lock")):
            os.remove(file)
        for file in glob(os.path.join(CONFIG_PATH, ".*.tmp")):
            os.remove(file)
        for file in glob(os.path.join(ROOT_DIR, "credentials", ".*.lock")):
            os.remove(file)
        for file in glob(os.path.join(ROOT_DIR, "credentials", ".*.tmp")):
            os.remove(file)
        if os.path.exists(os.path.join(CONFIG_PATH, "old_user_info.yml")):
            os.remove(os.path.join(CONFIG_PATH, "old_user_info.yml"))

    def test_get_legacy_config_path(self):
        from neon_utils.configuration_utils import _get_legacy_config_dir
        test_dir = join(CONFIG_PATH, "config_path_test_dirs")

        venv_path = join(test_dir, "arbitrary_venv")
        mycroft_path = join(test_dir, "default_mycroft")
        cloned_neon_path = join(test_dir, "cloned_neon")
        legacy_neon_path = join(test_dir, "legacy_neon_path")

        test_path = ["/lib/python3.8", "/usr/lib/python3.8", "/opt/mycroft"]
        self.assertIsNone(_get_legacy_config_dir(test_path))

        test_path.insert(0, f"{venv_path}/.venv/lib/python3.8/site-packages")
        self.assertEqual(_get_legacy_config_dir(test_path), venv_path)

        test_path.insert(0, mycroft_path)
        self.assertEqual(_get_legacy_config_dir(test_path), mycroft_path)

        test_path.insert(0, cloned_neon_path)
        self.assertEqual(_get_legacy_config_dir(test_path), cloned_neon_path)

        test_path.insert(0, legacy_neon_path)
        self.assertEqual(_get_legacy_config_dir(test_path),
                         f"{legacy_neon_path}/NGI")

        dev_test_path = ['', '/usr/lib/python38.zip', '/usr/lib/python3.8',
                         '/usr/lib/python3.8/lib-dynload',
                         f'{join(test_dir, "dev_environment")}/venv/lib/'
                         f'python3.8/site-packages',
                         f'{join(test_dir, "dev_environment")}/neon_cli',
                         f'{join(test_dir, "dev_environment")}/'
                         f'transcripts_controller',
                         f'{join(test_dir, "dev_environment")}/neon_enclosure',
                         f'{join(test_dir, "dev_environment")}/neon_speech',
                         f'{join(test_dir, "dev_environment")}/neon_audio',
                         f'{join(test_dir, "dev_environment")}/NeonCore',
                         f'{join(test_dir, "dev_environment")}/'
                         f'neon-test-utils',
                         f'{join(test_dir, "dev_environment")}/neon_display',
                         f'{join(test_dir, "dev_environment")}/'
                         f'neon_messagebus',
                         f'{join(test_dir, "dev_environment")}/neon_gui']
        self.assertEqual(_get_legacy_config_dir(dev_test_path),
                         f'{join(test_dir, "dev_environment")}/NeonCore')

    def test_get_config_dir(self):
        from neon_utils.configuration_utils import get_config_dir
        # default
        config_path = get_config_dir()
        self.assertTrue(os.path.isdir(config_path))

        # Valid override
        os.environ["OVOS_CONFIG_BASE_FOLDER"] = "neon"
        os.environ["XDG_CONFIG_HOME"] = expanduser("~/")
        config_path = get_config_dir()
        self.assertEqual(config_path, expanduser("~/neon"))
        self.assertTrue(os.path.isdir(config_path))
        os.environ.pop("XDG_CONFIG_HOME")
        os.environ.pop("OVOS_CONFIG_BASE_FOLDER")
        self.assertIsNone(os.getenv("XDG_CONFIG_HOME"))

    def test_delete_recursive_dictionary_keys(self):
        from neon_utils.configuration_utils import \
            delete_recursive_dictionary_keys

        # Delete Key
        test_dict = deepcopy(TEST_DICT)
        test_dict = delete_recursive_dictionary_keys(test_dict, ["key_1",
                                                                 "key1"])
        self.assertEqual(test_dict, {"section 1": {"key2": "val2"},
                                     "section 2": {"key_2": "val2"}})

        # Delete Section
        test_dict = deepcopy(TEST_DICT)
        test_dict = delete_recursive_dictionary_keys(test_dict, ["section 1"])
        self.assertEqual(test_dict, {"section 2": {"key_1": "val1",
                                                   "key_2": "val2"}})

    def test_dict_merge(self):
        from neon_utils.configuration_utils import dict_merge
        to_update = deepcopy(TEST_DICT)
        new_keys = {"section 2": {"key_2": "new2",
                                  "key_3": "val3"}}
        updated = dict_merge(to_update, new_keys)
        self.assertEqual(updated["section 2"], {"key_1": "val1",
                                                "key_2": "new2",
                                                "key_3": "val3"})

    def test_dict_make_equal_keys(self):
        from neon_utils.configuration_utils import dict_make_equal_keys

        # Simple
        to_update = deepcopy(TEST_DICT)
        new_keys = {"section 2": {"key_2": "new2",
                                  "key_3": "val3"}}
        updated = dict_make_equal_keys(to_update, new_keys)
        self.assertEqual(updated, {"section 2": {"key_2": "val2",
                                                 "key_3": "val3"}
                                   })
        # Depth 0 (top level only)
        to_update = deepcopy(TEST_DICT)
        new_keys = {"section 2": {"key_2": "new2",
                                  "key_3": "val3"}}
        updated = dict_make_equal_keys(to_update, new_keys, 0)
        self.assertEqual(updated, {"section 2": {"key_1": "val1",
                                                 "key_2": "val2"}
                                   })

        # Depth 1
        to_update = deepcopy(TEST_DICT)
        to_update["section 2"]["key_2"] = {"2_data": "value"}
        new_keys = {"section 2": {"key_2": {},
                                  "key_3": "val3"}}
        updated = dict_make_equal_keys(to_update, new_keys, 1)
        self.assertEqual(updated, {"section 2": {"key_2": {"2_data": "value"},
                                                 "key_3": "val3"}
                                   })

        # Empty dict (invalid request)
        to_update = deepcopy(TEST_DICT)
        new_keys = dict()
        with self.assertRaises(ValueError):
            dict_make_equal_keys(to_update, new_keys)

    def test_dict_update_keys(self):
        from neon_utils.configuration_utils import dict_update_keys
        to_update = deepcopy(TEST_DICT)
        new_keys = {"section 2": {"key_2": "new2",
                                  "key_3": "val3"}}
        updated = dict_update_keys(to_update, new_keys)
        self.assertEqual(updated["section 2"], {"key_1": "val1",
                                                "key_2": "val2",
                                                "key_3": "val3"})

    def test_write_json(self):
        from neon_utils.configuration_utils import write_to_json
        file_path = os.path.join(CONFIG_PATH, "test.json")
        write_to_json(TEST_DICT, file_path)
        with open(file_path, "r") as f:
            from_disk = json.load(f)
        self.assertEqual(from_disk, TEST_DICT)
        os.remove(file_path)

    def test_get_user_config_add_keys(self):
        from neon_utils.configuration_utils import get_neon_user_config
        old_user_info = os.path.join(CONFIG_PATH, "old_user_info.yml")
        ngi_user_info = os.path.join(CONFIG_PATH, "ngi_user_info.yml")
        bak_local_conf = os.path.join(CONFIG_PATH, "bak_local_conf.yml")
        ngi_local_conf = os.path.join(CONFIG_PATH, "ngi_local_conf.yml")

        shutil.copy(ngi_local_conf, bak_local_conf)
        shutil.copy(ngi_user_info, old_user_info)
        config = get_neon_user_config(CONFIG_PATH)
        user_config_keys = ["user", "brands", "speech", "units", "location"]
        self.assertTrue(all(k for k in user_config_keys if k in config))
        shutil.move(old_user_info, ngi_user_info)
        shutil.move(bak_local_conf, ngi_local_conf)

    def test_get_user_config_create(self):
        from neon_utils.configuration_utils import get_neon_user_config
        old_user_info = os.path.join(CONFIG_PATH, "old_user_info.yml")
        ngi_user_info = os.path.join(CONFIG_PATH, "ngi_user_info.yml")
        bak_local_conf = os.path.join(CONFIG_PATH, "bak_local_conf.yml")
        ngi_local_conf = os.path.join(CONFIG_PATH, "ngi_local_conf.yml")

        shutil.copy(ngi_local_conf, bak_local_conf)
        shutil.move(ngi_user_info, old_user_info)
        config = get_neon_user_config(CONFIG_PATH)
        self.assertTrue(os.path.isfile(ngi_user_info))
        user_config_keys = ["user", "brands", "speech", "units", "location"]
        self.assertTrue(all(k for k in user_config_keys if k in config))
        shutil.move(old_user_info, ngi_user_info)
        shutil.move(bak_local_conf, ngi_local_conf)

    def test_get_user_config_migrate_keys(self):
        from neon_utils.configuration_utils import get_neon_user_config, \
            NGIConfig
        bak_user_info = os.path.join(CONFIG_PATH, "bak_user_info.yml")
        ngi_user_info = os.path.join(CONFIG_PATH, "ngi_user_info.yml")
        old_user_info = os.path.join(CONFIG_PATH, "dep_user_info.yml")
        bak_local_conf = os.path.join(CONFIG_PATH, "bak_local_conf.yml")
        ngi_local_conf = os.path.join(CONFIG_PATH, "ngi_local_conf.yml")
        shutil.move(ngi_user_info, bak_user_info)
        shutil.copy(old_user_info, ngi_user_info)
        shutil.copy(ngi_local_conf, bak_local_conf)

        user_conf = get_neon_user_config(CONFIG_PATH)
        user_config_keys = ["user", "brands", "speech", "units", "location"]
        self.assertTrue(all(k for k in user_config_keys if k in user_conf))

        local_config_keys = ["speech", "interface", "listener", "skills",
                             "session", "tts", "stt", "logs", "device"]
        local_conf = NGIConfig("ngi_local_conf", CONFIG_PATH)
        self.assertTrue(all(k for k in local_config_keys if k in local_conf))

        shutil.move(bak_user_info, ngi_user_info)
        shutil.move(bak_local_conf, ngi_local_conf)

    def test_user_config_keep_keys(self):
        from neon_utils.configuration_utils import get_neon_user_config
        bak_user_info = os.path.join(CONFIG_PATH, "bak_user_info.yml")
        ngi_user_info = os.path.join(CONFIG_PATH, "ngi_user_info.yml")
        shutil.move(ngi_user_info, bak_user_info)

        user_conf = get_neon_user_config(CONFIG_PATH)
        user_conf.update_yaml_file("brands", "favorite_brands", {'neon': 1})
        self.assertEqual(user_conf["brands"]["favorite_brands"]['neon'], 1)

        new_user_conf = get_neon_user_config(CONFIG_PATH)
        self.assertEqual(user_conf.content, new_user_conf.content)
        self.assertEqual(user_conf["brands"]["favorite_brands"]['neon'], 1)

        shutil.move(bak_user_info, ngi_user_info)

    def test_get_mycroft_compat_config(self):
        from neon_utils.configuration_utils import \
            get_mycroft_compatible_config
        mycroft_config = get_mycroft_compatible_config()
        self.assertIsInstance(mycroft_config, dict)
        self.assertIsInstance(mycroft_config["gui_websocket"], dict)
        self.assertIsInstance(mycroft_config["gui_websocket"]["host"], str)
        self.assertIsInstance(mycroft_config["gui_websocket"]["base_port"],
                              int)
        self.assertIsInstance(mycroft_config["ready_settings"], list)
        self.assertIsInstance(mycroft_config['tts'], dict)
        self.assertIsInstance(mycroft_config["keys"], dict)
        # self.assertEqual(mycroft_config["skills"]["directory"],
        #                  mycroft_config["skills"]["directory_override"])
        self.assertIsInstance(mycroft_config["language"], dict)
        self.assertIsInstance(mycroft_config["listener"], dict)
        self.assertIsInstance(mycroft_config["stt"], dict)
        self.assertIsInstance(mycroft_config["tts"], dict)
        try:
            from typing import OrderedDict
            self.assertNotIsInstance(mycroft_config['tts'], OrderedDict)
            # self.assertNotIsInstance(mycroft_config['tts']['amazon'],
            #                          OrderedDict)
        except ImportError:
            pass

    def test_make_loaded_config_safe(self):
        from ruamel.yaml import YAML
        from neon_utils.configuration_utils import _make_loaded_config_safe
        test_config = join(dirname(__file__), "configuration",
                           "local_conf_with_stt_tts.yml")
        with open(test_config) as f:
            config = YAML().load(f)
        clean_config = _make_loaded_config_safe(config)
        self.assertIsInstance(clean_config, dict)
        try:
            from typing import OrderedDict
            self.assertNotIsInstance(clean_config['tts'], OrderedDict)
            self.assertNotIsInstance(clean_config['tts']['amazon'],
                                     OrderedDict)
        except ImportError:
            pass

    def test_is_neon_core(self):
        from neon_utils.configuration_utils import is_neon_core
        self.assertIsInstance(is_neon_core(), bool)

    def test_create_config_from_setup_params(self):
        from neon_utils.configuration_utils import \
            create_config_from_setup_params
        # devMode
        test_dir = f"{ROOT_DIR}/test_setup_config"
        os.environ["devMode"] = "true"
        os.environ["autoStart"] = "true"
        os.environ["autoUpdate"] = "false"
        os.environ["devName"] = "Test-Device"
        os.environ["sttModule"] = "stt_module"
        os.environ["ttsModule"] = "tts_module"
        os.environ["installServer"] = "false"
        os.environ["installerDir"] = test_dir
        os.environ["logsDir"] = test_dir
        os.environ["GITHUB_TOKEN"] = "git_token"
        config = create_config_from_setup_params(test_dir)

        self.assertTrue(config["debug"])
        self.assertFalse(config["skills"]["auto_update"])
        self.assertEqual(config["device_name"], "Test-Device")
        self.assertEqual(config["stt"]["module"], "stt_module")
        self.assertEqual(config["tts"]["module"], "tts_module")
        self.assertEqual(config["log_dir"], test_dir)
        with open(join(test_dir, "neon.yaml")) as f:
            disk_config = yaml.safe_load(f)
        self.assertEqual(config, disk_config)
        shutil.rmtree(test_dir)

    def test_write_mycroft_compatible_config(self):
        from neon_utils.configuration_utils import \
            get_mycroft_compatible_config, write_mycroft_compatible_config
        test_path = os.path.join(CONFIG_PATH, "test.conf")
        config = get_mycroft_compatible_config()
        write_mycroft_compatible_config(test_path)
        with open(test_path) as f:
            from_disk = json.load(f)
        self.assertEqual(from_disk, config)
        write_mycroft_compatible_config(test_path)
        with open(test_path) as f:
            from_disk_2 = json.load(f)
        self.assertEqual(from_disk_2, config)
        self.assertIsNone(config["Audio"].get("Audio"))
        os.remove(test_path)

    def test_config_no_permissions(self):
        from neon_utils.configuration_utils import NGIConfig
        with self.assertRaises(PermissionError):
            NGIConfig("test_config", "/root/")

    def test_parse_skill_configuration_valid(self):
        from neon_utils.configuration_utils import parse_skill_default_settings
        with open(join(CONFIG_PATH, "skill_settingsmeta.json")) as f:
            default_settings = json.load(f)
        parsed_settings = parse_skill_default_settings(default_settings)
        self.assertIsInstance(parsed_settings, dict)

    def test_populate_read_only_config_simple(self):
        from neon_utils.configuration_utils import NGIConfig
        from neon_utils.configuration_utils import _populate_read_only_config
        test_dir = os.path.join(ROOT_DIR, "configuration", "populate_tests")
        ro_dir = os.path.join(test_dir, "test_ro_dir")
        test_conf = NGIConfig("ngi_local_conf", test_dir, True)
        test_filename = basename(test_conf.file_path)

        self.assertTrue(_populate_read_only_config(ro_dir,
                                                   test_filename, test_conf))
        os.remove(test_conf.file_path)

    @unittest.skip('Deprecated function')
    @mock.patch('neon_utils.configuration_utils._init_ovos_conf')
    def test_init_config_dir(self, init_ovos_conf):
        from neon_utils.configuration_utils import init_config_dir
        ro_dir = os.path.join(ROOT_DIR, "configuration", "unwritable_path")
        config_dir = os.path.join(ROOT_DIR, "configuration", "test")
        os.environ["OVOS_CONFIG_BASE_FOLDER"] = "neon"
        os.environ["NEON_CONFIG_PATH"] = ro_dir
        os.environ["XDG_CONFIG_HOME"] = config_dir
        init_config_dir()

        # Test config migration
        # self.assertEqual(os.environ["NEON_CONFIG_PATH"],
        #                  join(config_dir, "neon"))
        self.assertFalse(os.path.exists(join(config_dir, "neon",
                                             "ngi_local_conf.yml")))
        self.assertTrue(os.path.exists(join(config_dir, "neon",
                                            "ngi_local_conf.bak")))
        self.assertTrue(os.path.exists(join(config_dir, "neon",
                                            "neon.yaml")))
        with open(join(config_dir, "neon", "neon.yaml")) as f:
            config = yaml.safe_load(f)
        with open(join(config_dir, "neon", "ngi_local_conf.bak")) as f:
            old_config = yaml.safe_load(f)
        self.assertEqual(config["MQ"], old_config["MQ"])

        init_ovos_conf.assert_called_once()
        shutil.rmtree(os.environ.pop("XDG_CONFIG_HOME"))
        os.environ.pop("NEON_CONFIG_PATH")
        os.environ.pop("OVOS_CONFIG_BASE_FOLDER")

    def test_get_mycroft_compatible_location(self):
        from neon_utils.configuration_utils import \
            get_mycroft_compatible_location, get_neon_user_config

        old_user_info = os.path.join(CONFIG_PATH, "old_user_info.yml")
        ngi_user_info = os.path.join(CONFIG_PATH, "ngi_user_info.yml")
        shutil.copy(ngi_user_info, old_user_info)

        user_config = get_neon_user_config(CONFIG_PATH)

        # with self.assertRaises(KeyError):
        #     get_mycroft_compatible_location(user_config.content)

        # Default mycroft.conf
        location = get_mycroft_compatible_location(user_config["location"])

        self.assertIsInstance(location["city"]["name"], str)
        self.assertIsInstance(location["city"]["code"], str)
        self.assertIsInstance(location["city"]["state"]["name"], str)
        self.assertIsInstance(location["city"]["state"]["code"], str)
        self.assertIsInstance(location["city"]["state"]["country"]["name"], str)
        self.assertIsInstance(location["city"]["state"]["country"]["code"], str)

        self.assertIsInstance(location["timezone"]["code"], str)
        self.assertIsInstance(location["coordinate"]["latitude"], float)
        self.assertIsInstance(location["coordinate"]["longitude"], float)
        self.assertIsInstance(location["timezone"]["name"], str)
        self.assertIsInstance(location["timezone"]["offset"], (float, int))
        self.assertEqual(location["timezone"]["dstOffset"], 3600000)

        # Valid user configured location
        user_config['location'] = {
            'lat': '47.4799078',
            'lng': '-122.2034496',
            'city': 'Renton',
            'state': 'Washington',
            'country': 'United States',
            'tz': 'America/Los_Angeles',
            'utc': '-8.0'
        }
        location = get_mycroft_compatible_location(user_config["location"])
        self.assertEqual(location["city"]["name"],
                         user_config["location"]["city"])
        self.assertEqual(location["city"]["code"],
                         user_config["location"]["city"])
        self.assertEqual(location["city"]["state"]["name"],
                         user_config["location"]["state"])
        self.assertIsInstance(location["city"]["state"]["code"], str)
        self.assertEqual(location["city"]["state"]["country"]["name"],
                         user_config["location"]["country"])
        self.assertEqual(location["city"]["state"]["country"]["code"], "us")

        self.assertIsInstance(location["coordinate"]["latitude"], float)
        self.assertEqual(str(location["coordinate"]["latitude"]),
                         user_config["location"]["lat"])
        self.assertIsInstance(location["coordinate"]["longitude"], float)
        self.assertEqual(str(location["coordinate"]["longitude"]),
                         user_config["location"]["lng"])

        self.assertEqual(location["timezone"]["code"],
                         user_config["location"]["tz"])
        self.assertIsInstance(location["timezone"]["name"], str)
        self.assertIsInstance(location["timezone"]["offset"], float)
        self.assertEqual(location["timezone"]["dstOffset"], 3600000)

        real_lat = user_config["location"]["lat"]
        real_lon = user_config["location"]["lng"]

        # Quoted strings in location
        user_config["location"]["lat"] = f'"{real_lat}"'
        user_config["location"]["lng"] = f'"{real_lon}"'
        user_config["location"]["utc"] = '-8.0'
        location = get_mycroft_compatible_location(user_config["location"])
        self.assertEqual(location["city"]["state"]["country"]["name"],
                         user_config["location"]["country"])
        self.assertEqual(location["coordinate"]["latitude"], float(real_lat))
        self.assertEqual(location["coordinate"]["longitude"], float(real_lon))
        self.assertEqual(location["city"]["state"]["country"]["code"], "us")

        self.assertIsInstance(location["coordinate"]["latitude"], float)
        self.assertIsInstance(location["coordinate"]["longitude"], float)
        self.assertIsInstance(location["timezone"]["offset"], float)

        user_config["location"]["lat"] = f"'{real_lat}'"
        user_config["location"]["lng"] = f"'{real_lon}'"
        location = get_mycroft_compatible_location(user_config["location"])
        self.assertIsInstance(location["coordinate"]["latitude"], float)
        self.assertIsInstance(location["coordinate"]["longitude"], float)
        user_config["location"]["utc"] = '"-8.0"'
        location = get_mycroft_compatible_location(user_config["location"])
        self.assertIsInstance(location["timezone"]["offset"], float)

        # Float
        user_config["location"]["utc"] = 8.0
        location = get_mycroft_compatible_location(user_config["location"])
        self.assertIsInstance(location["timezone"]["offset"], float)

        user_config["location"]["utc"] = ''
        location = get_mycroft_compatible_location(user_config["location"])
        self.assertIsInstance(location["timezone"]["offset"], float)

        # Incomplete user locations
        location = {
            'lat': None,
            'lng': None,
            'city': None,
            'state': None,
            'country': None,
            'tz': None,
            'utc': None
        }
        location['tz'] = 'America/Los_Angeles'
        self.assertIsInstance(get_mycroft_compatible_location(location), dict)
        location['tz'] = None
        location['city'] = 'Renton'
        self.assertIsInstance(get_mycroft_compatible_location(location), dict)
        location['city'] = None
        location['lat'] = '47.4799078'
        location['lng'] = '-122.2034496'
        self.assertIsInstance(get_mycroft_compatible_location(location), dict)

        # Empty Mycroft config location
        self.assertIsInstance(get_mycroft_compatible_location({}), dict)

        shutil.move(old_user_info, ngi_user_info)

    def test_get_user_config_from_mycroft_conf(self):
        from neon_utils.configuration_utils import \
            get_user_config_from_mycroft_conf
        config = get_user_config_from_mycroft_conf()
        self.assertIsInstance(config, dict)
        # TODO: Better tests of config load

    @unittest.skip('Internal function is deprecated')
    @mock.patch('neon_utils.packaging_utils.get_neon_core_root')
    def test_init_ovos_conf(self, get_core_root):
        default_config = join(dirname(__file__), "configuration", "neon_core")
        get_core_root.return_value = default_config
        test_config_dir = join(dirname(__file__), "test_config")
        from neon_utils.configuration_utils import _init_ovos_conf
        os.environ["XDG_CONFIG_HOME"] = test_config_dir

        if isfile(join(test_config_dir, "OpenVoiceOS", "ovos.conf")):
            os.remove(join(test_config_dir, "OpenVoiceOS", "ovos.conf"))

        # Init 'test_module' to use 'neon_core' config
        _init_ovos_conf("test_module")

        with open(join(test_config_dir, "OpenVoiceOS", "ovos.conf")) as f:
            config = json.load(f)

        # Patch local tests
        config['module_overrides']['neon_core'].setdefault(
            'default_config_path', "")

        self.assertEqual(config, {"module_overrides": {
            "neon_core": {
                "xdg": True,
                "base_folder": "neon",
                "config_filename": "neon.yaml",
                "default_config_path": config['module_overrides']['neon_core'][
                    'default_config_path']
            }
        },
            "submodule_mappings": {
                "test_module": "neon_core"
            }})

        # init same module again, config should be unchanged
        _init_ovos_conf("test_module")
        with open(join(test_config_dir, "OpenVoiceOS", "ovos.conf")) as f:
            config2 = json.load(f)
        # Patch local tests
        config2['module_overrides']['neon_core'].setdefault(
            'default_config_path', "")
        self.assertEqual(config, config2)

        # init another different module
        _init_ovos_conf("other_test_mod")
        with open(join(test_config_dir, "OpenVoiceOS", "ovos.conf")) as f:
            config3 = json.load(f)
            # Patch local tests
            config3['module_overrides']['neon_core'].setdefault(
                'default_config_path', "")
        self.assertEqual(config3, {"module_overrides": {
            "neon_core": {
                "xdg": True,
                "base_folder": "neon",
                "config_filename": "neon.yaml",
                "default_config_path": config['module_overrides']['neon_core'][
                    'default_config_path']
            }
        },
            "submodule_mappings": {
                "test_module": "neon_core",
                "other_test_mod": "neon_core"
            }})

        # init neon_core
        _init_ovos_conf("neon_core")
        with open(join(test_config_dir, "OpenVoiceOS", "ovos.conf")) as f:
            config4 = json.load(f)
            # Patch local tests
            config4['module_overrides']['neon_core'].setdefault(
                'default_config_path', "")
        self.assertEqual(config4, {"module_overrides": {
            "neon_core": {
                "xdg": True,
                "base_folder": "neon",
                "config_filename": "neon.yaml",
                "default_config_path": config['module_overrides']['neon_core'][
                    'default_config_path']
            }
        },
            "submodule_mappings": {
                "test_module": "neon_core",
                "other_test_mod": "neon_core",
                "neon_core": "neon_core",
                "neon_core.skills.skill_manager": "neon_core"
            }})

        # Override default config with test file
        import inspect
        import ovos_config.models
        import ovos_config.config
        from ovos_config.meta import get_ovos_config

        ovos_config.DEFAULT_CONFIG = join(dirname(__file__),
                                          "configuration", "mycroft.conf")
        old_value = deepcopy(ovos_config.DEFAULT_CONFIG)

        # Init config and validate other config file is loaded
        stack = inspect.stack()
        mod = inspect.getmodule(stack[1][0])
        this_modname = mod.__name__.split('.')[0]
        _init_ovos_conf(this_modname)
        self.assertNotEqual(old_value, ovos_config.DEFAULT_CONFIG)
        self.assertEqual(ovos_config.models.DEFAULT_CONFIG,
                         ovos_config.DEFAULT_CONFIG)
        self.assertEqual(ovos_config.config.Configuration.default.path,
                         ovos_config.DEFAULT_CONFIG)

        # Test default config
        self.assertTrue(ovos_config.config.Configuration()['default_config'])
        self.assertEqual(get_ovos_config()['default_config_path'],
                         join(default_config, "configuration", "neon.yaml"))

        # Cleanup configuration and force reload of pre-test defaults
        os.environ.pop("XDG_CONFIG_HOME")
        shutil.rmtree(test_config_dir)
        del ovos_config.config.Configuration
        importlib.reload(ovos_config.locations)
        importlib.reload(ovos_config.models)
        importlib.reload(ovos_config.config)

    def test_validate_config_env(self):
        from neon_utils.configuration_utils import _validate_config_env
        os.environ["XDG_CONFIG_HOME"] = ""
        os.environ["OVOS_CONFIG_BASE_FOLDER"] = "neon"

        # Valid default path
        _validate_config_env()
        self.assertEqual(os.environ["NEON_CONFIG_PATH"],
                         expanduser("~/.config/neon"))

        # Invalid neon spec
        os.environ["NEON_CONFIG_PATH"] = "/tmp"
        _validate_config_env()
        self.assertEqual(os.environ["NEON_CONFIG_PATH"],
                         expanduser("~/.config/neon"))

        # Valid neon spec
        os.environ["NEON_CONFIG_PATH"] = "/tmp/neon/neon"
        _validate_config_env()
        self.assertEqual(os.environ["NEON_CONFIG_PATH"], "/tmp/neon/neon")
        self.assertEqual(os.environ["XDG_CONFIG_HOME"], "/tmp/neon")

        # Valid XDG override, Valid Neon spec
        os.environ["XDG_CONFIG_HOME"] = "/tmp/xdg"
        _validate_config_env()
        self.assertEqual(os.environ["NEON_CONFIG_PATH"], "/tmp/xdg/neon")
        self.assertEqual(os.environ["XDG_CONFIG_HOME"], "/tmp/xdg")

        # Valid XDG override, No Neon spec
        os.environ["NEON_CONFIG_PATH"] = ""
        _validate_config_env()
        self.assertEqual(os.environ["NEON_CONFIG_PATH"], "/tmp/xdg/neon")
        self.assertEqual(os.environ["XDG_CONFIG_HOME"], "/tmp/xdg")

        # Valid XDG Override, relocate Neon-spec config
        os.environ["XDG_CONFIG_HOME"] = "/tmp/neon_config"
        os.environ["NEON_CONFIG_PATH"] = join(dirname(__file__),
                                              "test_migrate_config")
        _validate_config_env()
        self.assertEqual(os.environ["NEON_CONFIG_PATH"],
                         "/tmp/neon_config/neon")
        with open(join(dirname(__file__), "test_migrate_config",
                       "ngi_local_conf.yml")) as f:
            actual = f.read()
        with open("/tmp/neon_config/neon/ngi_local_conf.yml") as f:
            migrated = f.read()
        self.assertEqual(actual, migrated)
        shutil.rmtree("/tmp/neon_config")

        os.environ.pop("XDG_CONFIG_HOME")
        os.environ.pop("OVOS_CONFIG_BASE_FOLDER")

    def test_migrate_ngi_config(self):
        from neon_utils.configuration_utils import migrate_ngi_config
        test_dir = join(dirname(__file__), "test_migrate_config")
        new_conf = join(test_dir, "neon.yaml")
        os.environ["XDG_CONFIG_PATH"] = test_dir

        # Spec old config dir
        migrate_ngi_config(test_dir, join(test_dir, "neon.yaml"))
        self.assertTrue(isfile(new_conf))
        with open(join(test_dir, "ngi_local_conf.yml")) as f:
            try:
                old_config = yaml.safe_load(f)
            except Exception as e:
                LOG.error(e)
                from ruamel.yaml import YAML
                f.seek(0)
                old_config = json.loads(json.dumps(YAML().load(f)))
        with open(new_conf) as f:
            new_config = yaml.safe_load(f)
        last_change = getmtime(new_conf)
        self.assertEqual(new_config.get('device_name'),
                         old_config["devVars"]['devName'])
        self.assertEqual(new_config['hotwords'], old_config['hotwords'])
        for setting in old_config['listener']:
            self.assertEqual(old_config['listener'][setting],
                             new_config['listener'][setting])
        self.assertEqual(old_config['logs']['log_level'],
                         new_config['log_level'])
        for setting in old_config['stt']:
            self.assertEqual(old_config['stt'][setting],
                             new_config['stt'][setting])
        for setting in old_config['tts']:
            self.assertEqual(old_config['tts'][setting],
                             new_config['tts'][setting])
        self.assertEqual(old_config["MQ"], new_config["MQ"])

        # Spec old config file
        migrate_ngi_config(join(test_dir, "ngi_local_conf.yml"), new_conf)
        with open(new_conf) as f:
            newer_config = yaml.safe_load(f)
        self.assertNotEqual(last_change, getmtime(new_conf))
        self.assertEqual(new_config, newer_config)

        os.remove(new_conf)

    @mock.patch('neon_utils.packaging_utils.get_neon_core_root')
    def test_get_neon_yaml_config(self, get_core_root):
        config_dir = join(dirname(__file__), "configuration",
                          "get_neon_yaml_config")
        get_core_root.return_value = join(config_dir, "default")
        os.environ["XDG_CONFIG_HOME"] = config_dir
        os.environ["MYCROFT_SYSTEM_CONFIG"] = join(config_dir, "system",
                                                   "neon.yaml")
        from neon_utils.configuration_utils import _get_neon_yaml_config, \
            init_config_dir
        init_config_dir()
        config = _get_neon_yaml_config()
        self.assertEqual(config,
                         {"config": "user",
                          "from_default": True,
                          "from_system": True,
                          "user": {
                              "from_system": True,
                              "from_default": True,
                              "from_user": True,
                              "not_from_user": False
                          }})
        shutil.rmtree(join(config_dir, os.environ["OVOS_CONFIG_BASE_FOLDER"]))


class DeprecatedConfigTests(unittest.TestCase):
    def doCleanups(self) -> None:
        if os.getenv("NEON_CONFIG_PATH"):
            os.environ.pop("NEON_CONFIG_PATH")
        for file in glob(os.path.join(CONFIG_PATH, ".*.lock")):
            os.remove(file)
        for file in glob(os.path.join(CONFIG_PATH, ".*.tmp")):
            os.remove(file)
        for file in glob(os.path.join(ROOT_DIR, "credentials", ".*.lock")):
            os.remove(file)
        for file in glob(os.path.join(ROOT_DIR, "credentials", ".*.tmp")):
            os.remove(file)
        if os.path.exists(os.path.join(CONFIG_PATH, "old_user_info.yml")):
            os.remove(os.path.join(CONFIG_PATH, "old_user_info.yml"))

    def test_get_audio_config(self):
        from neon_utils.configuration_utils import _get_neon_audio_config
        config = _get_neon_audio_config()
        self.assertIsInstance(config, dict)
        self.assertIsInstance(config["Audio"], dict)
        self.assertIsInstance(config["tts"], dict)
        self.assertIsInstance(config["language"], dict)

    def test_get_gui_config(self):
        from neon_utils.configuration_utils import _get_neon_gui_config
        config = _get_neon_gui_config()
        self.assertIsInstance(config, dict)
        # self.assertIsInstance(config["lang"], str)
        # self.assertIsInstance(config["enclosure"], str)
        # self.assertIsInstance(config["host"], str)
        # self.assertIsInstance(config["port"], int)
        # self.assertIsInstance(config["base_port"], int)
        # self.assertIsInstance(config["route"], str)
        # self.assertIsInstance(config["ssl"], bool)
        # self.assertIsInstance(config["resource_root"], str)
        # self.assertIn("file_server", config.keys())
        # self.assertEqual(config["port"], config["base_port"])

    def test_get_lang_config(self):
        from neon_utils.configuration_utils import _get_neon_lang_config
        config = _get_neon_lang_config()
        self.assertIsInstance(config, dict)
        # self.assertIn("internal", config)
        # self.assertIn("user", config)
        # self.assertIn("detection_module", config)
        # self.assertIn("translation_module", config)
        # self.assertIn("boost", config)
        # self.assertIsInstance(config["libretranslate"], dict)

    def test_get_transcribe_config(self):
        from neon_utils.configuration_utils import _get_neon_transcribe_config
        config = _get_neon_transcribe_config()
        self.assertIsInstance(config, dict)
        self.assertIsInstance(config["audio_permission"], bool)
        self.assertIsInstance(config["transcript_dir"], str)

    def test_get_tts_config(self):
        from neon_utils.configuration_utils import _get_neon_tts_config
        config = _get_neon_tts_config()
        self.assertIsInstance(config, dict)
        # self.assertIsInstance(config["module"], str)
        # self.assertIsInstance(config[config["module"]], dict)

    def test_get_skills_config(self):
        from neon_utils.configuration_utils import _get_neon_skills_config
        config = _get_neon_skills_config()
        self.assertIsInstance(config["debug"], bool)
        self.assertIsInstance(config["blacklisted_skills"], list)
        self.assertIsInstance(config["priority_skills"], list)
        self.assertIsInstance(config["update_interval"], float)
        # self.assertIsInstance(config["data_dir"], str)
        # self.assertIsInstance(config["skill_manager"], str)

        # self.assertIsInstance(config["install_default"], bool)
        # self.assertIsInstance(config["install_essential"], bool)
        # self.assertIn("default_skills", config)
        # self.assertIn("essential_skills", config)
        # self.assertIn("neon_token", config)

        self.assertEqual(config["update_interval"],
                         config["auto_update_interval"])  # Backwards Compat.
        # self.assertIsInstance(config["directory"], str)
        # self.assertIsInstance(config["extra_directories"], list)
        # self.assertIsInstance(config["disable_osm"], bool)

        if config.get("msm"):
            self.assertIsInstance(config["msm"], dict)
            # self.assertIsInstance(config["msm"]["directory"], str)
            self.assertIsInstance(config["msm"]["versioned"], bool)
            self.assertIsInstance(config["msm"]["repo"], dict)

            self.assertIsInstance(config["msm"]["repo"]["branch"], str)
            self.assertIsInstance(config["msm"]["repo"]["cache"], str)
            self.assertIsInstance(config["msm"]["repo"]["url"], str)

    def test_added_module_config(self):
        from neon_utils.configuration_utils import _get_neon_local_config
        bak_local_conf = os.path.join(CONFIG_PATH, "ngi_local_conf.bak")
        ngi_local_conf = os.path.join(CONFIG_PATH, "ngi_local_conf.yml")
        ngi_test_conf = os.path.join(CONFIG_PATH,
                                     "local_conf_with_stt_tts.yml")

        shutil.move(ngi_local_conf, bak_local_conf)
        shutil.copy(ngi_test_conf, ngi_local_conf)
        local_config = _get_neon_local_config(CONFIG_PATH)
        self.assertEqual(local_config["tts"]["mozilla_remote"],
                         {"url": "http://something.somewhere"})
        self.assertEqual(local_config["stt"]["some_module"], {"key": "value"})
        # self.assertIn("dirVars", local_config.content.keys())
        shutil.move(bak_local_conf, ngi_local_conf)

    def test_move_language_config(self):
        from neon_utils.configuration_utils import _get_neon_local_config
        bak_local_conf = os.path.join(CONFIG_PATH, "ngi_local_conf.bak")
        ngi_local_conf = os.path.join(CONFIG_PATH, "ngi_local_conf.yml")
        ngi_test_conf = os.path.join(CONFIG_PATH, "local_conf_no_language.yml")

        shutil.move(ngi_local_conf, bak_local_conf)
        shutil.copy(ngi_test_conf, ngi_local_conf)
        local_config = _get_neon_local_config(CONFIG_PATH)
        self.assertEqual(local_config["language"]["translation_module"],
                         "old_translate_module")
        self.assertEqual(local_config["language"]["detection_module"],
                         "old_detection_module")
        # self.assertIsInstance(local_config["language"]["libretranslate"], dict)
        self.assertIn("dirVars", local_config.content.keys())
        shutil.move(bak_local_conf, ngi_local_conf)

    def test_added_hotwords_config(self):
        from neon_utils.configuration_utils import _get_neon_local_config
        bak_local_conf = os.path.join(CONFIG_PATH, "ngi_local_conf.bak")
        ngi_local_conf = os.path.join(CONFIG_PATH, "ngi_local_conf.yml")

        test_hotword_config = {"listen": True,
                               "model": "model_path"}

        shutil.move(ngi_local_conf, bak_local_conf)
        local_config = _get_neon_local_config(CONFIG_PATH)
        local_config.content.setdefault('hotwords', {})
        hotwords_config = deepcopy(local_config["hotwords"])
        local_config['hotwords']["test_hotword"] = test_hotword_config
        for hotword, config in hotwords_config.items():
            self.assertEqual(local_config['hotwords'][hotword], config)

        fresh_config = _get_neon_local_config(CONFIG_PATH)
        self.assertEqual(fresh_config.content, local_config.content)
        self.assertEqual(fresh_config['hotwords']['test_hotword'],
                         test_hotword_config)

        shutil.move(bak_local_conf, ngi_local_conf)

    def test_get_neon_auth_config(self):
        from neon_utils.authentication_utils import find_neon_aws_keys,\
            find_neon_google_keys, find_neon_git_token, find_neon_wolfram_key,\
            find_neon_alpha_vantage_key, find_neon_owm_key
        from neon_utils.configuration_utils import _get_neon_auth_config
        auth_path = os.path.join(ROOT_DIR, "credentials")
        ngi_auth_vars = _get_neon_auth_config(auth_path)
        self.assertEqual(ngi_auth_vars["amazon"],
                         find_neon_aws_keys(auth_path))
        self.assertEqual(ngi_auth_vars["google"],
                         find_neon_google_keys(auth_path))
        self.assertEqual(ngi_auth_vars["github"],
                         {"token": find_neon_git_token(auth_path)})
        self.assertEqual(ngi_auth_vars["wolfram"],
                         {"app_id": find_neon_wolfram_key(auth_path)})
        self.assertEqual(ngi_auth_vars["alpha_vantage"],
                         {"api_key": find_neon_alpha_vantage_key(auth_path)})
        self.assertEqual(ngi_auth_vars["owm"],
                         {"api_key": find_neon_owm_key(auth_path)})

    def test_get_neon_auth_config_unwritable(self):
        from neon_utils.configuration_utils import _get_neon_auth_config, \
            get_config_dir
        real_auth_config = join(get_config_dir(), "ngi_auth_vars.yml")
        bak_auth_config = join(get_config_dir(), "ngi_auth.bak")
        if isfile(real_auth_config):
            shutil.copy(real_auth_config, bak_auth_config)
        os.environ["NEON_CONFIG_PATH"] = os.path.join(ROOT_DIR,
                                                      "configuration",
                                                      "unwritable_path")
        ngi_auth_vars = _get_neon_auth_config()
        self.assertIsInstance(ngi_auth_vars, dict)

        if isfile(bak_auth_config):
            shutil.move(bak_auth_config, real_auth_config)

    def test_safe_mycroft_config(self):
        from neon_utils.configuration_utils import _safe_mycroft_config

        config = _safe_mycroft_config()
        self.assertIsInstance(config, dict)
        self.assertIn("skills", config)

    def test_populate_read_only_config_no_overwrite(self):
        from neon_utils.configuration_utils import _get_neon_local_config
        from neon_utils.configuration_utils import _populate_read_only_config
        test_dir = os.path.join(ROOT_DIR, "configuration", "populate_tests")
        ro_dir = os.path.join(test_dir, "test_ro_dir")
        test_conf = _get_neon_local_config(test_dir)
        test_filename = basename(test_conf.file_path)

        self.assertFalse(_populate_read_only_config(test_dir,
                                                    test_filename, test_conf))

        os.chdir(test_dir)
        self.assertFalse(_populate_read_only_config("./",
                                                    test_filename, test_conf))
        self.assertFalse(_populate_read_only_config(None,
                                                    test_filename, test_conf))
        self.assertFalse(_populate_read_only_config(ro_dir,
                                                    test_filename, test_conf))
        os.remove(test_conf.file_path)

    def test_default_config(self):
        from neon_utils.configuration_utils import _get_neon_local_config
        config = _get_neon_local_config("/tmp/neon/test/")
        self.assertIsInstance(config.content, dict)
        # import requests
        # resp = requests.get(config["skills"]["default_skills"])
        # self.assertTrue(resp.ok)

    def test_simultaneous_config_updates(self):
        from neon_utils.configuration_utils import _get_neon_lang_config, \
            _get_neon_local_config, get_neon_user_config
        from threading import Thread
        test_results = {}

        config_path = join(CONFIG_PATH, "depreciated_language_config")
        backup_path = join(CONFIG_PATH, "backup_config")
        os.environ["NEON_CONFIG_PATH"] = config_path
        if isdir(backup_path):
            shutil.rmtree(backup_path)
        shutil.copytree(config_path, backup_path)

        def _open_config(idx):
            success = True
            try:
                local_config = \
                    deepcopy(_get_neon_local_config(config_path).content)
                self.assertNotIn("translation_module", local_config["stt"])
                self.assertNotIn("detection_module", local_config["stt"])
            except Exception as e:
                LOG.error(e)
                success = False
            try:
                user_config = get_neon_user_config(config_path)
                self.assertNotIn("listener", user_config.content.keys())
            except Exception as e:
                LOG.error(e)
                success = False
            try:
                lang_config = _get_neon_lang_config()
                self.assertIsInstance(lang_config["boost"], bool)
            except Exception as e:
                LOG.error(e)
                success = False
            test_results[idx] = success

        for i in range(64):
            Thread(target=_open_config, args=(i,), daemon=True).start()
        while not len(test_results.keys()) == 64:
            sleep(0.5)
        self.assertTrue(all(test_results.values()))

        shutil.rmtree(config_path)
        shutil.move(backup_path, config_path)

    def test_unequal_cache_configs(self):
        from neon_utils.configuration_utils import _get_neon_local_config
        bak_local_conf = os.path.join(CONFIG_PATH, "ngi_local_conf.bak")
        ngi_local_conf = os.path.join(CONFIG_PATH, "ngi_local_conf.yml")
        shutil.copy(ngi_local_conf, bak_local_conf)

        def_config = _get_neon_local_config(f"{ROOT_DIR}/test")
        oth_config = _get_neon_local_config(CONFIG_PATH)
        self.assertNotEqual(def_config, oth_config)
        self.assertNotEqual(def_config.content, oth_config.content)

        if isdir(f"{ROOT_DIR}/test"):
            shutil.rmtree(f"{ROOT_DIR}/test")
        shutil.move(bak_local_conf, ngi_local_conf)


    def test_get_local_config_add_keys(self):
        from neon_utils.configuration_utils import _get_neon_local_config
        old_local_conf = os.path.join(CONFIG_PATH, "old_local_conf.yml")
        ngi_local_conf = os.path.join(CONFIG_PATH, "ngi_local_conf.yml")
        shutil.copy(ngi_local_conf, old_local_conf)
        config = _get_neon_local_config(CONFIG_PATH)
        local_config_keys = ["prefFlags", "interface", "devVars", "gestures",
                             "audioService", "padatious", "websocket",
                             "gui", "hotwords", "listener", "skills",
                             "session", "tts", "stt", "logs", "device"]
        self.assertTrue(all(k for k in local_config_keys if k in config))
        shutil.move(old_local_conf, ngi_local_conf)

    def test_get_local_config_create(self):
        from neon_utils.configuration_utils import _get_neon_local_config
        old_local_conf = os.path.join(CONFIG_PATH, "old_local_conf.yml")
        ngi_local_conf = os.path.join(CONFIG_PATH, "ngi_local_conf.yml")
        shutil.move(ngi_local_conf, old_local_conf)
        config = _get_neon_local_config(CONFIG_PATH)
        local_config_keys = ["prefFlags", "interface", "devVars", "gestures",
                             "audioService", "padatious", "websocket",
                             "gui", "hotwords", "listener", "skills",
                             "session", "tts", "stt", "logs", "device"]
        self.assertTrue(all(k for k in local_config_keys if k in config))
        shutil.move(old_local_conf, ngi_local_conf)


if __name__ == '__main__':
    unittest.main()



# NEON AI (TM) SOFTWARE, Software Development Kit & Application Framework
# All trademark and other rights reserved by their respective owners
# Copyright 2008-2025 Neongecko.com Inc.
# Contributors: Daniel McKnight, Guy Daniels, Elon Gasper, Richard Leeds,
# Regina Bloomstine, Casimiro Ferreira, Andrii Pernatii, Kirill Hrymailo
# BSD-3 License
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS;  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE,  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
import sys
import unittest

from threading import Event
from os.path import join, dirname

import ovos_utils.signal
from ovos_bus_client import Message
from ovos_utils.messagebus import FakeBus

sys.path.append(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
import neon_utils.signal_utils


class TestSignalManager:
    def __init__(self, bus):
        self.bus = bus
        self._register_listeners()
        self.bus.run_in_thread()

    def _register_listeners(self):
        """
        Register Event Handlers
        """
        self.bus.on("neon.create_signal", self._handle_create_signal)
        self.bus.on("neon.check_for_signal", self._handle_check_for_signal)
        self.bus.on("neon.wait_for_signal_create", self._handle_wait_for_signal_create)
        self.bus.on("neon.wait_for_signal_clear", self._handle_wait_for_signal_clear)
        self.bus.on("neon.signal_manager_active", self._handle_signal_manager_active)

    def _handle_create_signal(self, message: Message):
        signal_name = message.data["signal_name"]
        status = True
        self.bus.emit(message.reply(f"neon.create_signal.{signal_name}",
                                    data={"signal_name": signal_name,
                                          "is_set": status}))

    def _handle_check_for_signal(self, message: Message):
        signal_name = message.data["signal_name"]
        status = True
        self.bus.emit(message.reply(f"neon.check_for_signal.{signal_name}",
                                    data={"signal_name": signal_name,
                                          "is_set": status}))

    def _handle_wait_for_signal_create(self, message: Message):
        signal_name = message.data["signal_name"]
        status = True
        self.bus.emit(message.reply(f"neon.wait_for_signal_create.{signal_name}",
                                    data={"signal_name": signal_name,
                                          "is_set": status}))

    def _handle_wait_for_signal_clear(self, message: Message):
        signal_name = message.data["signal_name"]
        status = False
        self.bus.emit(message.reply(f"neon.wait_for_signal_clear.{signal_name}",
                                    data={"signal_name": signal_name,
                                          "is_set": status}))

    def _handle_signal_manager_active(self, message: Message):
        self.bus.emit(message.response())


class SignalUtilsTests(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        test_ipc_dir = join(dirname(__file__), "signal_tests")
        cls.test_config = {"ipc_path": test_ipc_dir}
        # cls.manager = None

    def setUp(self):
        self.test_bus = FakeBus()
        self.test_bus.connected_event = Event()
        self.test_bus.connected_event.set()
        self.test_bus.run_in_thread()
        neon_utils.signal_utils.init_signal_bus(self.test_bus)
        neon_utils.signal_utils.init_signal_handlers()

    def test_signal_file(self):
        self.assertTrue(neon_utils.signal_utils.create_signal("test", self.test_config))
        self.assertTrue(neon_utils.signal_utils.check_for_signal("test", -1, self.test_config))
        self.assertTrue(neon_utils.signal_utils.check_for_signal("test", 0, self.test_config))
        self.assertFalse(neon_utils.signal_utils.check_for_signal("test", -1, self.test_config))

    def test_check_signal_manager_available(self):
        self.assertFalse(neon_utils.signal_utils.check_signal_manager_available())

        TestSignalManager(self.test_bus)
        self.assertTrue(neon_utils.signal_utils.check_signal_manager_available())

    def test_signal_manager_message_context(self):
        msg = None

        def on_create(message):
            nonlocal msg
            msg = message

        TestSignalManager(self.test_bus)
        neon_utils.signal_utils.init_signal_handlers()
        self.test_bus.once("neon.create_signal", on_create)
        from neon_utils.signal_utils import create_signal
        create_signal("test_signal")
        self.assertIsInstance(msg, Message)
        self.assertEqual(msg.data, {'signal_name': 'test_signal'})
        self.assertEqual(msg.context['origin_module'],
                         'tests.signal_util_tests')
        self.assertEqual(msg.context['origin_line'], 130)

    def test_signal_utils_manager_available(self):
        TestSignalManager(self.test_bus)
        neon_utils.signal_utils.init_signal_handlers()
        self.assertEqual(neon_utils.signal_utils._BUS, self.test_bus)
        self.assertIsInstance(neon_utils.signal_utils._MAX_TIMEOUT, int)
        self.assertTrue(neon_utils.signal_utils.check_signal_manager_available())

        # Check neon_utils references
        self.assertEqual(neon_utils.signal_utils._check_for_signal,
                         neon_utils.signal_utils._manager_check_for_signal)
        self.assertEqual(neon_utils.signal_utils._create_signal,
                         neon_utils.signal_utils._manager_create_signal)
        self.assertEqual(neon_utils.signal_utils._wait_for_signal_clear,
                         neon_utils.signal_utils._manager_wait_for_signal_clear)
        self.assertEqual(neon_utils.signal_utils._wait_for_signal_create,
                         neon_utils.signal_utils._manager_wait_for_signal_create)

        # Check ovos_utils references
        self.assertEqual(ovos_utils.signal.check_for_signal,
                         neon_utils.signal_utils._manager_check_for_signal)
        self.assertEqual(ovos_utils.signal.create_signal,
                         neon_utils.signal_utils._manager_create_signal)

    def test_signal_utils_manager_unavailable(self):
        import ovos_utils.signal
        neon_utils.signal_utils.init_signal_handlers()
        self.assertFalse(neon_utils.signal_utils.check_signal_manager_available())
        self.assertIsInstance(neon_utils.signal_utils._MAX_TIMEOUT, int)
        self.assertEqual(neon_utils.signal_utils._check_for_signal,
                         ovos_utils.signal.check_for_signal)
        self.assertEqual(neon_utils.signal_utils._create_signal,
                         ovos_utils.signal.create_signal)
        self.assertEqual(neon_utils.signal_utils._wait_for_signal_clear,
                         neon_utils.signal_utils._fs_wait_for_signal_clear)
        self.assertEqual(neon_utils.signal_utils._wait_for_signal_create,
                         neon_utils.signal_utils._fs_wait_for_signal_create)

    def test_signal_utils_reload(self):
        import ovos_utils.signal
        from neon_utils.signal_utils import _check_for_signal
        self.assertFalse(neon_utils.signal_utils.check_signal_manager_available())
        self.assertEqual(neon_utils.signal_utils._check_for_signal,
                         ovos_utils.signal.check_for_signal)
        self.assertEqual(neon_utils.signal_utils._create_signal,
                         ovos_utils.signal.create_signal)
        self.assertEqual(neon_utils.signal_utils._wait_for_signal_clear,
                         neon_utils.signal_utils._fs_wait_for_signal_clear)
        self.assertEqual(neon_utils.signal_utils._wait_for_signal_create,
                         neon_utils.signal_utils._fs_wait_for_signal_create)
        self.assertEqual(_check_for_signal,
                         ovos_utils.signal.check_for_signal)

        TestSignalManager(self.test_bus)
        neon_utils.signal_utils.init_signal_handlers()
        self.assertEqual(neon_utils.signal_utils._check_for_signal,
                         neon_utils.signal_utils._manager_check_for_signal)
        self.assertEqual(neon_utils.signal_utils._create_signal,
                         neon_utils.signal_utils._manager_create_signal)
        self.assertEqual(neon_utils.signal_utils._wait_for_signal_clear,
                         neon_utils.signal_utils._manager_wait_for_signal_clear)
        self.assertEqual(neon_utils.signal_utils._wait_for_signal_create,
                         neon_utils.signal_utils._manager_wait_for_signal_create)

        # Previously imported method is not updated
        self.assertEqual(_check_for_signal,
                         ovos_utils.signal.check_for_signal)

    def test_check_signal_manager_available_lazy_load_bus(self):
        from ovos_bus_client import MessageBusClient
        from neon_utils.signal_utils import check_signal_manager_available
        neon_utils.signal_utils._BUS = None
        neon_utils.signal_utils.check_signal_manager_available()
        self.assertIsInstance(neon_utils.signal_utils._BUS, MessageBusClient)
        self.assertIsInstance(neon_utils.signal_utils._MAX_TIMEOUT, int)


if __name__ == '__main__':
    unittest.main()



# NEON AI (TM) SOFTWARE, Software Development Kit & Application Framework
# All trademark and other rights reserved by their respective owners
# Copyright 2008-2025 Neongecko.com Inc.
# Contributors: Daniel McKnight, Guy Daniels, Elon Gasper, Richard Leeds,
# Regina Bloomstine, Casimiro Ferreira, Andrii Pernatii, Kirill Hrymailo
# BSD-3 License
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS;  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE,  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
import sys
import unittest

sys.path.append(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
from neon_utils.search_utils import *


CON_DICT = {"cattalk.com,12345,Cat Talk Convo": ["cat", "talk", "convo"],
            "dogreport.com,54321,Dog Report Convo": ["dog", "report", "convo"]}

DOM_DICT = {"cattalk.com": ["cat", "cats", "cattalk", "cat talk"],
            "dogreport.com": ["dog", "dogs", "dogreport", "dog report"]}

MSG_DICT = {"cattalk.com,12345,0,username,sid,how are you doing": ["how", "are", "you", "doing"],
            "dogreport.com,54321,0,username,sid,I love my dog": ["i", "love", "my", "dog"]}


class SearchUtilTests(unittest.TestCase):
    def test_search_con(self):
        results = search_convo_dict(CON_DICT, "cat")
        self.assertIn("cattalk.com,12345,Cat Talk Convo", results)

        results = search_convo_dict(CON_DICT, "convo")
        self.assertIn("cattalk.com,12345,Cat Talk Convo", results)
        self.assertIn("dogreport.com,54321,Dog Report Convo", results)

    def test_search_dom(self):
        results = search_convo_dict(DOM_DICT, "cat")
        self.assertIn("cattalk.com", results)
        self.assertNotIn("dogreport.com", results)

    def test_search_shout(self):
        results = search_convo_dict(MSG_DICT, "dog")
        self.assertIn("dogreport.com,54321,0,username,sid,I love my dog", results)

    # # TODO: Need to document typo handling/search expected results DM
    # def test_search_typo(self):
    #     results = search_convo_dict(MSG_DICT, "dig", handle_typos=True)
    #     self.assertIn("dogreport.com,54321,0,username,sid,I love my dog", results)
    #
    #     results = search_convo_dict(MSG_DICT, "dig", handle_typos=False)
    #     self.assertNotIn("dogreport.com,54321,0,username,sid,I love my dog", results)


if __name__ == '__main__':
    unittest.main()



# NEON AI (TM) SOFTWARE, Software Development Kit & Application Framework
# All trademark and other rights reserved by their respective owners
# Copyright 2008-2025 Neongecko.com Inc.
# Contributors: Daniel McKnight, Guy Daniels, Elon Gasper, Richard Leeds,
# Regina Bloomstine, Casimiro Ferreira, Andrii Pernatii, Kirill Hrymailo
# BSD-3 License
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS;  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE,  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import unittest
import sys
import os

sys.path.append(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
from neon_utils.parse_utils import *


class ParseUtilTests(unittest.TestCase):
    def test_clean_quotes_simple(self):
        raw_str = '"this is a double-quoted-string"'
        output = clean_quotes(raw_str)
        self.assertEqual(output, "this is a double-quoted-string")

        raw_str = 'this has a trailing double quote"'
        output = clean_quotes(raw_str)
        self.assertEqual(raw_str, output)

        raw_str = '"this has a leading double quote'
        output = clean_quotes(raw_str)
        self.assertEqual(raw_str, output)

        raw_str = "'this is a single-quoted string'"
        output = clean_quotes(raw_str)
        self.assertEqual(output, "this is a single-quoted string")

    def test_clean_quotes_foreign(self):
        raw_str = '「this has Japanese quotes」'
        output = clean_quotes(raw_str)
        self.assertEqual(output, "this has Japanese quotes")

        raw_str = "«this has French quotes»"
        output = clean_quotes(raw_str)
        self.assertEqual(output, "this has French quotes")

    def test_clean_quotes_error_null(self):
        with self.assertRaises(ValueError):
            clean_quotes(None)

    def test_clean_quotes_error_type(self):
        with self.assertRaises(TypeError):
            clean_quotes(["list"])
        with self.assertRaises(TypeError):
            clean_quotes(123)

    def test_normalize_spoken_string_valid(self):
        valid_string = "hello."
        self.assertEqual(normalize_string_to_speak(valid_string), valid_string)
        valid_string = "hello?"
        self.assertEqual(normalize_string_to_speak(valid_string), valid_string)
        valid_string = "hello!"
        self.assertEqual(normalize_string_to_speak(valid_string), valid_string)
        valid_string = "hello..."
        self.assertEqual(normalize_string_to_speak(valid_string), valid_string)

    def test_normalize_spoken_string_add_punctuation(self):
        invalid_string = "hello"
        self. assertEqual(normalize_string_to_speak(invalid_string), f"{invalid_string}.")

    def test_normalize_spoken_string_error_null(self):
        with self.assertRaises(ValueError):
            normalize_string_to_speak(None)

    def test_normalize_spoken_string_error_type(self):
        with self.assertRaises(TypeError):
            normalize_string_to_speak(["list"])
        with self.assertRaises(TypeError):
            normalize_string_to_speak(123)

    def test_clean_filename(self):
        raw_filename = "'My*Weird~Filename I want to use?__'"
        cleaned = clean_filename(raw_filename)
        self.assertEqual(cleaned, "'My_Weird_Filename I want to use___'")
        lowered = clean_filename(raw_filename, True)
        self.assertEqual(lowered, cleaned.lower())

    def test_clean_transcription(self):
        raw_input = "50% is acceptable-ish. Right?"
        cleaned_input = clean_transcription(raw_input)
        self.assertEqual(cleaned_input, "50 percent is acceptable ish  right")

    def test_get_phonemes(self):
        wake_word = "Hey Neon"
        phonemes = get_phonemes(wake_word)
        self.assertEqual(phonemes, "HH EY . N IY AA N .")

        phonemes = get_phonemes("okay")
        self.assertEqual(phonemes, "OW K EY .")

    def test_format_speak_tags_with_speech(self):
        valid_output = "<speak>Speak This.</speak>"
        no_tags = format_speak_tags("Speak This.")
        self.assertEqual(no_tags, valid_output)

        leading_only = format_speak_tags("<speak>Speak This.")
        self.assertEqual(leading_only, valid_output)

        leading_with_exclusion = format_speak_tags("Nope.<speak>Speak This.")
        self.assertEqual(leading_with_exclusion, valid_output)

        trailing_only = format_speak_tags("Speak This.</speak>")
        self.assertEqual(trailing_only, valid_output)

        trailing_with_exclusion = format_speak_tags("Speak This.</speak> But not this.")
        self.assertEqual(trailing_with_exclusion, valid_output)

        tagged = format_speak_tags("<speak>Speak This.</speak>")
        self.assertEqual(tagged, valid_output)

        tagged_with_exclusion = format_speak_tags("Don't<speak>Speak This.</speak>But Not this.")
        self.assertEqual(tagged_with_exclusion, valid_output)

    def test_format_speak_tags_empty(self):
        leading_closure = format_speak_tags("</speak>hello.")
        self.assertFalse(leading_closure)

        trailing_open = format_speak_tags("hello.<speak>")
        self.assertFalse(trailing_open)

    def test_format_speak_tags_with_speech_no_tags(self):
        valid_output = "Speak This."
        no_tags = format_speak_tags("Speak This.", False)
        self.assertEqual(no_tags, valid_output)

        leading_only = format_speak_tags("<speak>Speak This.", False)
        self.assertEqual(leading_only, valid_output)

        leading_with_exclusion = format_speak_tags("Nope.<speak>Speak This.", False)
        self.assertEqual(leading_with_exclusion, valid_output)

        trailing_only = format_speak_tags("Speak This.</speak>", False)
        self.assertEqual(trailing_only, valid_output)

        trailing_with_exclusion = format_speak_tags("Speak This.</speak> But not this.", False)
        self.assertEqual(trailing_with_exclusion, valid_output)

        tagged = format_speak_tags("<speak>Speak This.</speak>", False)
        self.assertEqual(tagged, valid_output)

        tagged_with_exclusion = format_speak_tags("Don't<speak>Speak This.</speak>But Not this.", False)
        self.assertEqual(tagged_with_exclusion, valid_output)

    def test_validate_email(self):
        from neon_utils.parse_utils import validate_email
        valid = "developers@neon.ai"
        self.assertEqual(validate_email(valid), valid)
        self.assertEqual(validate_email(f" {valid} "), valid)
        self.assertEqual(validate_email("developers @ neon. ai"), valid)

        self.assertEqual(validate_email(" not an email address"), "")
        self.assertEqual(validate_email("test@neon"), "")
        self.assertEqual(validate_email("test@neon. "), "")
        self.assertEqual(validate_email("neon.ai"), "")


if __name__ == '__main__':
    unittest.main()



# NEON AI (TM) SOFTWARE, Software Development Kit & Application Framework
# All trademark and other rights reserved by their respective owners
# Copyright 2008-2025 Neongecko.com Inc.
# Contributors: Daniel McKnight, Guy Daniels, Elon Gasper, Richard Leeds,
# Regina Bloomstine, Casimiro Ferreira, Andrii Pernatii, Kirill Hrymailo
# BSD-3 License
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS;  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE,  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import json
import time
from collections import OrderedDict


class LRUCache:
    # TODO user specific cache with a compound key
    def __init__(self, capacity: int = 128):
        self.cache = OrderedDict()
        self._capacity = capacity
        self._hits = 0
        self._missed = 0
        self._init_time = time.time()

    def __len__(self):
        return len(self.cache)

    @property
    def hits(self):
        return self._hits

    @property
    def missed(self):
        return self._missed

    def get(self, key):
        """
        Return the value of the key from cache.
        Increment self._missed in key not in cache, increment self._hits in key in cache
        Args:
            key: a key to look up in cache
        Returns: value associated with the key or None

        """
        if key not in self.cache:
            self._missed += 1
            return None
        else:
            self._hits += 1
            self.cache.move_to_end(key)
            return self.cache[key]

    def put(self, key, value) -> None:
        """
        Put a key-value pair into cache
        Args:
            key: a key to put into cache
            value: a value to put into cache
        Returns:
        """
        self.cache[key] = value
        self.cache.move_to_end(key)
        if len(self.cache) > self._capacity:
            self.cache.popitem(last=False)

    def clear(self):
        """
        Clear the cache dict
        """
        self.cache = OrderedDict()

    def clear_full(self):
        """
        Clear the cache dict, reset hits, missed and initialization time
        Returns:

        """
        self.clear()
        self._hits = self._missed = 0
        self._init_time = time.time()

    def jsonify(self):
        """
        Dump cache dict into json
        Returns: json-string
        """
        return json.dumps(self.cache)

    def jsonify_metrics(self):
        """
        Dump cache metrics into json
        Returns:

        """
        data = dict()
        data["cache"] = self.cache
        data["hits"] = self.hits
        data["missed"] = self.missed
        data["size"] = self._capacity
        return json.dumps(data)



# NEON AI (TM) SOFTWARE, Software Development Kit & Application Framework
# All trademark and other rights reserved by their respective owners
# Copyright 2008-2025 Neongecko.com Inc.
# Contributors: Daniel McKnight, Guy Daniels, Elon Gasper, Richard Leeds,
# Regina Bloomstine, Casimiro Ferreira, Andrii Pernatii, Kirill Hrymailo
# BSD-3 License
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS;  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE,  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from os import getcwd, chdir
from os.path import isfile, dirname, abspath, join
from mock import Mock, patch
from importlib.util import spec_from_file_location, module_from_spec

from ovos_utils.log import LOG


def get_skill_metadata(skill_dir: str) -> dict:
    """
    Get a metadata object for a skill in a given directory.
    """
    setup_py = join(skill_dir, "setup.py")
    pyproject_toml = join(skill_dir, "pyproject.toml")
    readme_md = join(skill_dir, "README.md")

    if isfile(pyproject_toml):
        meta = _get_skill_data_poetry(pyproject_toml)
    elif isfile(setup_py):
        meta = _get_skill_data_setuptools(setup_py)
    elif isfile(readme_md):
        with open(readme_md, encoding="utf-8") as f:
            readme_data = f.read()
        meta = _get_skill_data_readme(readme_data)
    else:
        raise FileNotFoundError(
            f"No setup.py or pyproject.toml found in {skill_dir}"
        )

    # Adding params for backwards-compat.
    meta["desktopFile"] = False
    meta["warning"] = ""
    meta["systemDeps"] = False
    meta.setdefault("requirements", {})
    meta["requirements"].setdefault("system", {})
    meta["requirements"].setdefault("skill", [])
    meta.setdefault("incompatible_skills", [])
    meta["platforms"] = ["i386", "x86_64", "ia64", "arm64", "arm"]
    meta["branch"] = "master"
    meta["foldername"] = None
    meta.setdefault("short_description", meta.get("summary"))

    return meta


def _get_skill_data_poetry(pyproject: str) -> dict:
    """
    Get skill metadata from a pyproject.toml file
    Args:
        pyproject: pyproject.toml file to evaluate
    Returns:
        dict: Skill metadata
    """
    from toml import load

    if not isfile(pyproject):
        raise FileNotFoundError(f"Not a Directory: {pyproject}")
    with open(pyproject, encoding="utf-8") as f:
        data = load(f)
    skill_data = {
        "package_name": data["tool"]["poetry"].get("name", "Unknown"),
        "name": data["tool"]["poetry"].get("name", "Unknown"),
        "description": data["tool"]["poetry"].get("name", "description"),
        "pip_spec": data["tool"]["poetry"].get("name", "Unknown"),
        "license": data["tool"]["poetry"].get("license", "Unknown"),
        "author": data["tool"]["poetry"].get(
            "authors", [""]
        ),  # List of authors
        "tags": data["tool"]["poetry"].get("keywords", []),
        "version": data["tool"]["poetry"].get("version", ""),
    }
    print(data["tool"]["poetry"]["plugins"])
    skill_data["skill_id"] = list(
        data["tool"]["poetry"]
        .get("plugins", {})
        .get("ovos.plugin.skill", {})
        .keys()
    )[0].strip('"')

    # Below match existing Neon skill metadata
    skill_data["title"] = skill_data["name"]
    skill_data["summary"] = skill_data["description"]
    skill_data["short_description"] = skill_data["description"]
    skill_data["credits"] = [skill_data["author"]]
    skill_data["skillname"] = skill_data["name"]

    python_requirements = data["tool"]["poetry"].get("dependencies", {})
    # Parse requirements into expected string format
    python_requirements.pop("python", None)  # Remove python version spec
    skill_data["requirements"] = {
        "python": [f"{k}{v}" for k, v in python_requirements.items()]
    }

    if data["tool"]["poetry"].get("readme"):
        readme_path = join(
            dirname(pyproject), data["tool"]["poetry"]["readme"]
        )
        with open(readme_path) as f:
            readme_data = f.read()
        try:
            readme_metadata = _get_skill_data_readme(readme_data)
        except Exception as e:
            LOG.error(f"Failed to parse README for {skill_data['name']}: {e}")
            readme_metadata = {}
    return {**readme_metadata, **skill_data}


def _get_skill_data_setuptools(setup_py: str) -> dict:
    """
    Get skill metadata from a setup.py file by executing it with a mocked setuptools.setup
    Args:
        setup_py: setup.py file to evaluate
    Returns:
        dict: Skill metadata
    """

    if not isfile(setup_py):
        raise FileNotFoundError(f"File not found: {setup_py}")

    # Create a mock for setuptools.setup
    setup_mock = Mock()

    # Mock setuptools.find_packages as well
    find_packages_mock = Mock(return_value=[])

    # Change to the directory containing setup.py to handle relative paths
    original_cwd = getcwd()
    setup_dir = dirname(abspath(setup_py))

    try:
        chdir(setup_dir)

        # Mock setuptools imports and execute setup.py
        with patch.dict(
            "sys.modules",
            {
                "setuptools": Mock(
                    setup=setup_mock, find_packages=find_packages_mock
                )
            },
        ):
            # Load and execute the setup.py file
            spec = spec_from_file_location("setup", setup_py)
            setup_module = module_from_spec(spec)
            spec.loader.exec_module(setup_module)

        # Get the captured kwargs from the mock
        if not setup_mock.called:
            raise ValueError(f"No setup() call found in {setup_py}")

        captured_kwargs = (
            setup_mock.call_args.kwargs
            if setup_mock.call_args.kwargs
            else setup_mock.call_args.args[0]
            if setup_mock.call_args.args
            else {}
        )

    finally:
        chdir(original_cwd)

    # Build skill_data matching the expected format
    skill_data = {
        "package_name": captured_kwargs.get("name"),
        "pip_spec": captured_kwargs.get("name"),
        "license": captured_kwargs.get("license"),
        "author": captured_kwargs.get("author"),
        "tags": captured_kwargs.get("keywords", []),
        "version": captured_kwargs.get("version"),
        "url": captured_kwargs.get("url"),
    }

    # Extract skill_id from entry_points if available
    skill_data["skill_id"] = captured_kwargs["entry_points"][
        "ovos.plugin.skill"
    ]

    if "github.com" in skill_data["url"]:
        author, skill = skill_data["url"].split("github.com/")[1].split("/")
        skill_data["skillname"] = skill
        skill_data["authorname"] = author

    # Set additional fields to match existing Neon skill metadata format
    skill_data["name"] = skill_data["package_name"]
    skill_data["title"] = skill_data["package_name"]
    skill_data["credits"] = (
        [skill_data["author"]] if skill_data["author"] != "Unknown" else []
    )

    # Handle requirements
    install_requires = captured_kwargs.get("install_requires", [])
    skill_data["requirements"] = {
        "python": install_requires
        if isinstance(install_requires, list)
        else []
    }

    if "long_description" in captured_kwargs:
        readme_data = _get_skill_data_readme(
            captured_kwargs["long_description"]
        )
    else:
        readme_data = {}
    return {**readme_data, **skill_data}


def _get_skill_data_readme(readme_md: str) -> dict:
    """
    Get skill metadata from a README file
    Args:
        readme_md: README.md file contents to evaluate
    Returns:
        dict: Skill metadata
    """
    from neon_utils.parse_utils import clean_quotes

    lines = readme_md.split("\n")
    if not lines or len(lines) <= 1:
        raise ValueError("Empty README data")

    # Initialize parser params
    list_sections = (
        "examples",
        "incompatible skills",
        "platforms",
        "categories",
        "tags",
        "credits",
    )
    valid_sections = list_sections + (
        "summary",
        "short_description",
        "description",
        "warning",
    )
    section = "header"
    category = None
    parsed_data = {}

    def _check_section_start(ln: str):
        # Handle section start
        if ln.startswith(
            "# ![](https://0000.us/klatchat/app/files/"
            "neon_images/icons/neon_paw.png)"
        ):
            # Old style title line
            parsed_data["title"] = ln.split(")", 1)[1].strip()
            parsed_data["icon"] = ln.split("(", 1)[1].split(")", 1)[0].strip()
            return
        elif section == "header" and ln.startswith("# <img src="):
            # Title line
            parsed_data["title"] = ln.split(">", 1)[1].strip()
            parsed_data["icon"] = (
                ln.split("src=", 1)[1]
                .split()[0]
                .strip('"')
                .strip("'")
                .lstrip("./")
            )
            return "summary"
        elif ln.startswith("# ") or ln.startswith("## "):
            # Top-level section
            if ln.startswith("## About"):
                # Handle Mycroft 'About' section
                return "description"
            elif ln.startswith("## Category"):
                # Handle 'Category' as 'Categories'
                return "categories"
            else:
                return line.lstrip("#").strip().lower()
        return

    def _format_readme_line(ln: str):
        nonlocal category
        if section == "incompatible skills":
            if not any((ln.startswith("-"), ln.startswith("*"))):
                return None
            parsed = clean_quotes(ln.lstrip("-").lstrip("*").lower().strip())
            if parsed.startswith("["):
                return parsed.split("(", 1)[1].split(")", 1)[0]
            return parsed
        if section == "examples":
            if not any((ln.startswith("-"), ln.startswith("*"))):
                return None
            parsed = clean_quotes(ln.lstrip("-").lstrip("*").strip())
            if parsed.split(maxsplit=1)[0].lower() == "neon":
                return parsed.split(maxsplit=1)[1]
            else:
                return parsed
        if section == "categories":
            parsed = ln.rstrip("\n").strip("*")
            if ln.startswith("**"):
                category = parsed
            return parsed
        if section == "credits":
            if ln.strip().startswith("["):
                return ln.split("[", 1)[1].split("]", 1)[0]
            return ln.rstrip("\n").lstrip("@")
        if section == "tags":
            return ln.lstrip("#").rstrip("\n")
        if section in list_sections:
            return clean_quotes(ln.lstrip("-").lstrip("*").lower().strip())
        return ln.rstrip("\n").rstrip()

    for line in lines:
        new_section = _check_section_start(line)
        if new_section:
            section = new_section
        elif line.strip():
            parsed_line = _format_readme_line(line)
            if not parsed_line:
                # Nothing to parse in this line
                continue
            if section in list_sections:
                if section not in parsed_data:
                    parsed_data[section] = list()
                parsed_data[section].append(parsed_line)
            else:
                if section not in valid_sections:
                    continue
                elif section not in parsed_data:
                    parsed_data[section] = parsed_line
                else:
                    parsed_data[section] = " ".join(
                        (parsed_data[section], parsed_line)
                    )
    parsed_data["category"] = (
        category or parsed_data.get("categories", [""])[0]
    )
    if parsed_data.get("incompatible skills"):
        parsed_data["incompatible_skills"] = parsed_data.pop(
            "incompatible skills"
        )
    if parsed_data.get("credits") and len(parsed_data["credits"]) == 1:
        parsed_data["credits"] = parsed_data["credits"][0].split(" ")

    return parsed_data



# NEON AI (TM) SOFTWARE, Software Development Kit & Application Framework
# All trademark and other rights reserved by their respective owners
# Copyright 2008-2025 Neongecko.com Inc.
# Contributors: Daniel McKnight, Guy Daniels, Elon Gasper, Richard Leeds,
# Regina Bloomstine, Casimiro Ferreira, Andrii Pernatii, Kirill Hrymailo
# BSD-3 License
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS;  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE,  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
import sys
import random
import unittest

sys.path.append(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
from neon_utils.cache_utils import *


class CacheUtilTests(unittest.TestCase):

    def setUp(self) -> None:
        self.lru_cache = LRUCache()

    # def tearDown(self) -> None:
    #     self.lru_cache.clear()

    def test_put(self):
        max_size = self.lru_cache._capacity

        self.lru_cache.put("1", 1)
        self.assertEqual(len(self.lru_cache), 1)

        for i in range(2, max_size+2):
            self.lru_cache.put(str(i), i)
        self.assertEqual(len(self.lru_cache), max_size)
        self.assertNotIn("1", self.lru_cache.cache)

        random_i = random.randint(0, max_size)
        self.lru_cache.put(str(random_i), random_i)
        self.assertEqual(random_i, self.lru_cache.cache.popitem()[1])

    def test_get(self):
        hits = self.lru_cache.hits
        missed = self.lru_cache.missed
        key = random.randint(0, self.lru_cache._capacity - 1)
        invalid_key = "a"

        for i in range(self.lru_cache._capacity):
            self.lru_cache.put(str(i), i)

        for i in range(self.lru_cache._capacity):
            self.assertIsNotNone(self.lru_cache.get(str(i)))

        self.assertIsNotNone(self.lru_cache.get('0'))
        self.assertIsNotNone(self.lru_cache.get(str(self.lru_cache._capacity - 1)))
        self.assertIsNone(self.lru_cache.get(str(self.lru_cache._capacity)))
        result = self.lru_cache.get(str(key))
        self.assertIsNotNone(result)
        self.assertEqual(result, key)

        self.assertEqual(self.lru_cache.hits, hits + 3 + self.lru_cache._capacity)
        self.assertEqual(self.lru_cache.cache.popitem()[1], key)

        self.assertIsNone(self.lru_cache.get(invalid_key))
        self.assertEqual(self.lru_cache.missed, missed + 2)

    def test_clear(self):
        for i in range(self.lru_cache._capacity):
            self.lru_cache.put(str(i), i)

        self.lru_cache.clear()
        self.assertEqual(len(self.lru_cache), 0)

    def test_clear_full(self):
        for i in range(self.lru_cache._capacity):
            self.lru_cache.put(str(i), i)
        self.lru_cache._missed = self.lru_cache._hits = 10
        init_time = self.lru_cache._init_time

        self.lru_cache.clear_full()

        self.assertEqual(len(self.lru_cache), 0)
        self.assertEqual(self.lru_cache.hits, 0)
        self.assertEqual(self.lru_cache.missed, 0)
        self.assertNotEqual(init_time, self.lru_cache._init_time)

    def test_jsonify(self):
        for i in range(self.lru_cache._capacity):
            self.lru_cache.put(str(i), i)

        with self.assertRaises(TypeError):
            json.dumps(self.lru_cache)

        j = self.lru_cache.jsonify()
        self.assertIsInstance(j, str)

    def jsonify_metrics(self):
        j = self.lru_cache.jsonify_metrics()
        self.assertIsInstance(j, str)


if __name__ == '__main__':
    unittest.main()



# NEON AI (TM) SOFTWARE, Software Development Kit & Application Framework
# All trademark and other rights reserved by their respective owners
# Copyright 2008-2025 Neongecko.com Inc.
# Contributors: Daniel McKnight, Guy Daniels, Elon Gasper, Richard Leeds,
# Regina Bloomstine, Casimiro Ferreira, Andrii Pernatii, Kirill Hrymailo
# BSD-3 License
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS;  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE,  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from ovos_bus_client import Message

from neon_utils.skills import CommonMessageSkill, CommonPlaySkill,\
    CommonQuerySkill, NeonFallbackSkill, NeonSkill, PatchedMycroftSkill,\
    InstructorSkill, KioskSkill

import ovos_workshop.skills.ovos
ovos_workshop.skills.ovos.MycroftSkill = PatchedMycroftSkill
from ovos_workshop.skills.fallback import FallbackSkill


class TestCMS(CommonMessageSkill):
    def __init__(self):
        super(TestCMS, self).__init__(name="Test Common Message Skill")

    def CMS_match_message_phrase(self, request, context):
        pass

    def CMS_handle_send_message(self, message):
        pass

    def CMS_match_call_phrase(self, contact, context):
        pass

    def CMS_handle_place_call(self, message):
        pass


class TestCPS(CommonPlaySkill):
    def __init__(self):
        super(TestCPS, self).__init__(name="Test Common Play Skill")

    def CPS_match_query_phrase(self, phrase, message):
        pass

    def CPS_start(self, phrase, data, message=None):
        pass


class TestCQS(CommonQuerySkill):
    def __init__(self):
        super(TestCQS, self).__init__(name="Test Common Query Skill")

    def CQS_match_query_phrase(self, phrase, message):
        pass


class TestFBS(NeonFallbackSkill):
    def __init__(self):
        super(TestFBS, self).__init__(name="Test Fallback Skill")


class TestNeonSkill(NeonSkill):
    def __init__(self):
        super(TestNeonSkill, self).__init__(name="Test Neon Skill")


class TestPatchedSkill(PatchedMycroftSkill):
    def __init__(self):
        super(TestPatchedSkill, self).__init__(name="Test Mycroft Skill")


class TestInstructorSkill(InstructorSkill):
    def __init__(self):
        super(TestInstructorSkill, self).__init__(name="Test Instructor Skill")

    def _access_data_source(self, *args, **kwargs):
        pass

    def _search_in_data_source(self, *args, **kwargs):
        pass

    def _get_instructions(self, *args, **kwargs):
        pass


class TestMycroftFallbackSkill(FallbackSkill):
    """
    Test case for the Mycroft HomeAssistantSkill
    """
    def __init__(self):
        from neon_utils.skills.mycroft_skill import PatchedMycroftSkill as MycroftSkill
        MycroftSkill.__init__(self)
        super().__init__(name="TestSkill")


class TestChatSkill(NeonSkill):
    from neon_utils.skills import chat_handler

    def __init__(self):
        super().__init__(name="Test Neon Chat Skill")

    @chat_handler("Test Bot")
    def handle_chat_message(self, message: Message):
        return message.data.get("test_response")


class TestKioskSkill(KioskSkill):
    def __init__(self):
        super(TestKioskSkill, self).__init__()
        self._timeout_seconds = 60

    @property
    def timeout_seconds(self) -> int:
        return self._timeout_seconds

    @property
    def greeting_dialog(self) -> str:
        return "greeting"

    @property
    def goodbye_dialog(self) -> str:
        return "goodbye"

    @property
    def timeout_dialog(self) -> str:
        return "timeout"

    @property
    def error_dialog(self) -> str:
        return "error"

    def setup_new_interaction(self, message: Message) -> bool:
        pass

    def handle_new_interaction(self, message: Message):
        pass

    def handle_end_interaction(self, message: Message):
        pass

    def handle_user_utterance(self, message: Message):
        pass
